
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _RANGE_ADDRESS_H_
#define _RANGE_ADDRESS_H_

#pragma once

#include <string>
#include <atlbase.h>
#include <atltypes.h>
#include "AddressItem.h"

namespace AutomateIT
{
    namespace CalcItems
    {
        class CRangeAddress
        {
        public:
            CRangeAddress(const std::wstring& name);
            CRangeAddress(const CPoint& topLeft, const CPoint& bottomRight);

            const CPoint& GetTopLeft() const;
            const CPoint& GetBottomRight() const;

            std::wstring ToString() const;

            long GetLeft() const;
            long GetRight() const;
            long GetTop() const;
            long GetBottom() const;

            bool IsRows() const;
            bool IsColumns() const;

            long GetCount() const;

            CRangeAddress& operator += (const CPoint& point);

            virtual ~CRangeAddress();

        public:

            inline static std::wstring GetSpot()
            {
                return std::wstring(&spot_, &spot_ + 1);
            }

        private:

            void ConstructByName(const std::wstring& name);

        private:

            CAddressItemPtr topLeft_;
            CAddressItemPtr bottomRight_;
            std::wstring sheetName_;
            static const wchar_t spot_ = L':';
            static const wchar_t ex_ = L'!';

        };
    }
}


#endif // _RANGE_ADDRESS_H_
