/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _APPLICATION_H_
#define _APPLICATION_H_

#include "Workbooks.h"
#include "DispatchHook.h"
#include "IDispatchWrapper.h"
#include "OOApplication.h"

/*
	CApplication class provides functionality of the MS Excel Application object by using Open Office.
	It implements IApplication interface of the MS Excel.
*/

namespace AutomateIT
{
	namespace ExcelClone
	{
		class ATL_NO_VTABLE CApplication : 
			public CComObjectRootEx<CComSingleThreadModel>,
			public CComCoClass<CApplication, &CLSID_Application>,
			public CDispatchHook< IDispatchImpl<_Application, &IID__Application, &LIBID_ExcelCloneLib, 1, 0> >
		{

		public:
			CApplication();
			virtual ~CApplication();

			DECLARE_REGISTRY_RESOURCEID(IDR_EXCELAPPLICATION)

			BEGIN_COM_MAP(CApplication)
				COM_INTERFACE_ENTRY(_Application)
				COM_INTERFACE_ENTRY(IDispatch)
			END_COM_MAP()

			DECLARE_PROTECT_FINAL_CONSTRUCT()

		public:			

            HRESULT STDMETHODCALLTYPE get_Application( 
                 _Application **RHS);

            HRESULT STDMETHODCALLTYPE get_Creator( 
                 XlCreator *RHS);

            HRESULT STDMETHODCALLTYPE get_Parent( 
                 Application **RHS);

            HRESULT STDMETHODCALLTYPE get_ActiveCell( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_ActiveChart( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ActiveDialog( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ActiveMenuBar( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ActivePrinter( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_ActivePrinter( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_ActiveSheet( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ActiveWindow( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ActiveWorkbook( 
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE get_AddIns( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Assistant( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Calculate( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Cells( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_Charts( 
                 Sheets **RHS);

            HRESULT STDMETHODCALLTYPE get_Columns( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_CommandBars( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_DDEAppReturnCode( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE DDEExecute( 
                 long Channel,
                 BSTR String,
                 long lcid);

            HRESULT STDMETHODCALLTYPE DDEInitiate( 
                 BSTR App,
                 BSTR Topic,
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE DDEPoke( 
                 long Channel,
                 VARIANT Item,
                 VARIANT Data,
                 long lcid);

            HRESULT STDMETHODCALLTYPE DDERequest( 
                 long Channel,
                 BSTR Item,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE DDETerminate( 
                 long Channel,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_DialogSheets( 
                 Sheets **RHS);

            HRESULT STDMETHODCALLTYPE Evaluate( 
                 VARIANT Name,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE _Evaluate( 
                 VARIANT Name,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE ExecuteExcel4Macro( 
                 BSTR String,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Intersect( 
                 Range *Arg1,
                 Range *Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT Arg16,
                 VARIANT Arg17,
                 VARIANT Arg18,
                 VARIANT Arg19,
                 VARIANT Arg20,
                 VARIANT Arg21,
                 VARIANT Arg22,
                 VARIANT Arg23,
                 VARIANT Arg24,
                 VARIANT Arg25,
                 VARIANT Arg26,
                 VARIANT Arg27,
                 VARIANT Arg28,
                 VARIANT Arg29,
                 VARIANT Arg30,
                 long lcid,
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_MenuBars( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Modules( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Names( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Range( 
                 VARIANT Cell1,
                 VARIANT Cell2,
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_Rows( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE Run( 
                 VARIANT Macro,
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT Arg16,
                 VARIANT Arg17,
                 VARIANT Arg18,
                 VARIANT Arg19,
                 VARIANT Arg20,
                 VARIANT Arg21,
                 VARIANT Arg22,
                 VARIANT Arg23,
                 VARIANT Arg24,
                 VARIANT Arg25,
                 VARIANT Arg26,
                 VARIANT Arg27,
                 VARIANT Arg28,
                 VARIANT Arg29,
                 VARIANT Arg30,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE _Run2( 
                 VARIANT Macro,
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT Arg16,
                 VARIANT Arg17,
                 VARIANT Arg18,
                 VARIANT Arg19,
                 VARIANT Arg20,
                 VARIANT Arg21,
                 VARIANT Arg22,
                 VARIANT Arg23,
                 VARIANT Arg24,
                 VARIANT Arg25,
                 VARIANT Arg26,
                 VARIANT Arg27,
                 VARIANT Arg28,
                 VARIANT Arg29,
                 VARIANT Arg30,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_Selection( 
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE SendKeys( 
                 VARIANT Keys,
                 VARIANT Wait,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Sheets( 
                 Sheets **RHS);

            HRESULT STDMETHODCALLTYPE get_ShortcutMenus( 
                 long Index,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ThisWorkbook( 
                 long lcid,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE get_Toolbars( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Union( 
                 Range *Arg1,
                 Range *Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT Arg16,
                 VARIANT Arg17,
                 VARIANT Arg18,
                 VARIANT Arg19,
                 VARIANT Arg20,
                 VARIANT Arg21,
                 VARIANT Arg22,
                 VARIANT Arg23,
                 VARIANT Arg24,
                 VARIANT Arg25,
                 VARIANT Arg26,
                 VARIANT Arg27,
                 VARIANT Arg28,
                 VARIANT Arg29,
                 VARIANT Arg30,
                 long lcid,
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_Windows( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Workbooks( 
                 Workbooks **RHS);

            HRESULT STDMETHODCALLTYPE get_WorksheetFunction( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Worksheets( 
                 Sheets **RHS);

            HRESULT STDMETHODCALLTYPE get_Excel4IntlMacroSheets( 
                 Sheets **RHS);

            HRESULT STDMETHODCALLTYPE get_Excel4MacroSheets( 
                 Sheets **RHS);

            HRESULT STDMETHODCALLTYPE ActivateMicrosoftApp( 
                 XlMSApplication Index,
                 long lcid);

            HRESULT STDMETHODCALLTYPE AddChartAutoFormat( 
                 VARIANT Chart,
                 BSTR Name,
                 VARIANT Description,
                 long lcid);

            HRESULT STDMETHODCALLTYPE AddCustomList( 
                 VARIANT ListArray,
                 VARIANT ByRow,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_AlertBeforeOverwriting( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_AlertBeforeOverwriting( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_AltStartupPath( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_AltStartupPath( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_AskToUpdateLinks( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_AskToUpdateLinks( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_EnableAnimations( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableAnimations( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_AutoCorrect( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Build( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_CalculateBeforeSave( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_CalculateBeforeSave( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Calculation( 
                 long lcid,
                 XlCalculation *RHS);

            HRESULT STDMETHODCALLTYPE put_Calculation( 
                 long lcid,
                 XlCalculation RHS);

            HRESULT STDMETHODCALLTYPE get_Caller( 
                 VARIANT Index,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_CanPlaySounds( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_CanRecordSounds( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_Caption( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_Caption( 
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_CellDragAndDrop( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_CellDragAndDrop( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE CentimetersToPoints( 
                 double Centimeters,
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE CheckSpelling( 
                 BSTR Word,
                 VARIANT CustomDictionary,
                 VARIANT IgnoreUppercase,
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_ClipboardFormats( 
                 VARIANT Index,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayClipboardWindow( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayClipboardWindow( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_ColorButtons( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ColorButtons( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_CommandUnderlines( 
                 long lcid,
                 XlCommandUnderlines *RHS);

            HRESULT STDMETHODCALLTYPE put_CommandUnderlines( 
                 long lcid,
                 XlCommandUnderlines RHS);

            HRESULT STDMETHODCALLTYPE get_ConstrainNumeric( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ConstrainNumeric( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE ConvertFormula( 
                 VARIANT Formula,
                 XlReferenceStyle FromReferenceStyle,
                 VARIANT ToReferenceStyle,
                 VARIANT ToAbsolute,
                 VARIANT RelativeTo,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_CopyObjectsWithCells( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_CopyObjectsWithCells( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Cursor( 
                 long lcid,
                 XlMousePointer *RHS);

            HRESULT STDMETHODCALLTYPE put_Cursor( 
                 long lcid,
                 XlMousePointer RHS);

            HRESULT STDMETHODCALLTYPE get_CustomListCount( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_CutCopyMode( 
                 long lcid,
                 XlCutCopyMode *RHS);

            HRESULT STDMETHODCALLTYPE put_CutCopyMode( 
                 long lcid,
                 XlCutCopyMode RHS);

            HRESULT STDMETHODCALLTYPE get_DataEntryMode( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_DataEntryMode( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE Dummy1( 
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy2( 
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy3( 
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy4( 
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy5( 
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy6( 
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy7( 
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy8( 
                 VARIANT Arg1,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy9( 
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Dummy10( 
                 VARIANT arg,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE Dummy11( void);

            HRESULT STDMETHODCALLTYPE get__Default( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_DefaultFilePath( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_DefaultFilePath( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE DeleteChartAutoFormat( 
                 BSTR Name,
                 long lcid);

            HRESULT STDMETHODCALLTYPE DeleteCustomList( 
                 long ListNum,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Dialogs( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayAlerts( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayAlerts( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayFormulaBar( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayFormulaBar( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayFullScreen( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayFullScreen( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayNoteIndicator( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayNoteIndicator( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayCommentIndicator( 
                 XlCommentDisplayMode *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayCommentIndicator( 
                 XlCommentDisplayMode RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayExcel4Menus( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayExcel4Menus( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayRecentFiles( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayRecentFiles( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayScrollBars( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayScrollBars( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayStatusBar( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayStatusBar( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE DoubleClick( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_EditDirectlyInCell( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EditDirectlyInCell( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_EnableAutoComplete( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableAutoComplete( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_EnableCancelKey( 
                 long lcid,
                 XlEnableCancelKey *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableCancelKey( 
                 long lcid,
                 XlEnableCancelKey RHS);

            HRESULT STDMETHODCALLTYPE get_EnableSound( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableSound( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_EnableTipWizard( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableTipWizard( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_FileConverters( 
                 VARIANT Index1,
                 VARIANT Index2,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_FileSearch( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_FileFind( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE _FindFile( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_FixedDecimal( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_FixedDecimal( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_FixedDecimalPlaces( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_FixedDecimalPlaces( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE GetCustomListContents( 
                 long ListNum,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE GetCustomListNum( 
                 VARIANT ListArray,
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE GetOpenFilename( 
                 VARIANT FileFilter,
                 VARIANT FilterIndex,
                 VARIANT Title,
                 VARIANT ButtonText,
                 VARIANT MultiSelect,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE GetSaveAsFilename( 
                 VARIANT InitialFilename,
                 VARIANT FileFilter,
                 VARIANT FilterIndex,
                 VARIANT Title,
                 VARIANT ButtonText,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE Goto( 
                 VARIANT Reference,
                 VARIANT Scroll,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Height( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE put_Height( 
                 long lcid,
                 double RHS);

            HRESULT STDMETHODCALLTYPE Help( 
                 VARIANT HelpFile,
                 VARIANT HelpContextID,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_IgnoreRemoteRequests( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_IgnoreRemoteRequests( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE InchesToPoints( 
                 double Inches,
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE InputBox( 
                 BSTR Prompt,
                 VARIANT Title,
                 VARIANT Default,
                 VARIANT Left,
                 VARIANT Top,
                 VARIANT HelpFile,
                 VARIANT HelpContextID,
                 VARIANT Type,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_Interactive( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_Interactive( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_International( 
                 VARIANT Index,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_Iteration( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_Iteration( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_LargeButtons( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_LargeButtons( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Left( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE put_Left( 
                 long lcid,
                 double RHS);

            HRESULT STDMETHODCALLTYPE get_LibraryPath( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE MacroOptions( 
                 VARIANT Macro,
                 VARIANT Description,
                 VARIANT HasMenu,
                 VARIANT MenuText,
                 VARIANT HasShortcutKey,
                 VARIANT ShortcutKey,
                 VARIANT Category,
                 VARIANT StatusBar,
                 VARIANT HelpContextID,
                 VARIANT HelpFile,
                 long lcid);

            HRESULT STDMETHODCALLTYPE MailLogoff( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE MailLogon( 
                 VARIANT Name,
                 VARIANT Password,
                 VARIANT DownloadNewMail,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_MailSession( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_MailSystem( 
                 long lcid,
                 XlMailSystem *RHS);

            HRESULT STDMETHODCALLTYPE get_MathCoprocessorAvailable( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_MaxChange( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE put_MaxChange( 
                 long lcid,
                 double RHS);

            HRESULT STDMETHODCALLTYPE get_MaxIterations( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_MaxIterations( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_MemoryFree( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_MemoryTotal( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_MemoryUsed( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_MouseAvailable( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_MoveAfterReturn( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_MoveAfterReturn( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_MoveAfterReturnDirection( 
                 long lcid,
                 XlDirection *RHS);

            HRESULT STDMETHODCALLTYPE put_MoveAfterReturnDirection( 
                 long lcid,
                 XlDirection RHS);

            HRESULT STDMETHODCALLTYPE get_RecentFiles( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Name( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE NextLetter( 
                 long lcid,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE get_NetworkTemplatesPath( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_ODBCErrors( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ODBCTimeout( 
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_ODBCTimeout( 
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_OnCalculate( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnCalculate( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnData( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnData( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnDoubleClick( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnDoubleClick( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnEntry( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnEntry( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE OnKey( 
                 BSTR Key,
                 VARIANT Procedure,
                 long lcid);

            HRESULT STDMETHODCALLTYPE OnRepeat( 
                 BSTR Text,
                 BSTR Procedure,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_OnSheetActivate( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnSheetActivate( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnSheetDeactivate( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnSheetDeactivate( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE OnTime( 
                 VARIANT EarliestTime,
                 BSTR Procedure,
                 VARIANT LatestTime,
                 VARIANT Schedule,
                 long lcid);

            HRESULT STDMETHODCALLTYPE OnUndo( 
                 BSTR Text,
                 BSTR Procedure,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_OnWindow( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnWindow( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OperatingSystem( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_OrganizationName( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_Path( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_PathSeparator( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_PreviousSelections( 
                 VARIANT Index,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_PivotTableSelection( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_PivotTableSelection( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_PromptForSummaryInfo( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_PromptForSummaryInfo( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Quit( void);

            HRESULT STDMETHODCALLTYPE RecordMacro( 
                 VARIANT BasicCode,
                 VARIANT XlmCode,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_RecordRelative( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_ReferenceStyle( 
                 long lcid,
                 XlReferenceStyle *RHS);

            HRESULT STDMETHODCALLTYPE put_ReferenceStyle( 
                 long lcid,
                 XlReferenceStyle RHS);

            HRESULT STDMETHODCALLTYPE get_RegisteredFunctions( 
                 VARIANT Index1,
                 VARIANT Index2,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE RegisterXLL( 
                 BSTR Filename,
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE Repeat( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE ResetTipWizard( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_RollZoom( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_RollZoom( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Save( 
                 VARIANT Filename,
                 long lcid);

            HRESULT STDMETHODCALLTYPE SaveWorkspace( 
                 VARIANT Filename,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_ScreenUpdating( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ScreenUpdating( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE SetDefaultChart( 
                 VARIANT FormatName,
                 VARIANT Gallery);

            HRESULT STDMETHODCALLTYPE get_SheetsInNewWorkbook( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_SheetsInNewWorkbook( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_ShowChartTipNames( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ShowChartTipNames( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_ShowChartTipValues( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ShowChartTipValues( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_StandardFont( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_StandardFont( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_StandardFontSize( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE put_StandardFontSize( 
                 long lcid,
                 double RHS);

            HRESULT STDMETHODCALLTYPE get_StartupPath( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_StatusBar( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE put_StatusBar( 
                 long lcid,
                 VARIANT RHS);

            HRESULT STDMETHODCALLTYPE get_TemplatesPath( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_ShowToolTips( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ShowToolTips( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Top( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE put_Top( 
                 long lcid,
                 double RHS);

            HRESULT STDMETHODCALLTYPE get_DefaultSaveFormat( 
                 XlFileFormat *RHS);

            HRESULT STDMETHODCALLTYPE put_DefaultSaveFormat( 
                 XlFileFormat RHS);

            HRESULT STDMETHODCALLTYPE get_TransitionMenuKey( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_TransitionMenuKey( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_TransitionMenuKeyAction( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_TransitionMenuKeyAction( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_TransitionNavigKeys( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_TransitionNavigKeys( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Undo( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_UsableHeight( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE get_UsableWidth( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE get_UserControl( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_UserControl( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_UserName( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_UserName( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_Value( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_VBE( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Version( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_Visible( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_Visible( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Volatile( 
                 VARIANT Volatile,
                 long lcid);

            HRESULT STDMETHODCALLTYPE _Wait( 
                 VARIANT Time,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Width( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE put_Width( 
                 long lcid,
                 double RHS);

            HRESULT STDMETHODCALLTYPE get_WindowsForPens( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_WindowState( 
                 long lcid,
                 XlWindowState *RHS);

            HRESULT STDMETHODCALLTYPE put_WindowState( 
                 long lcid,
                 XlWindowState RHS);

            HRESULT STDMETHODCALLTYPE get_UILanguage( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_UILanguage( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_DefaultSheetDirection( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_DefaultSheetDirection( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_CursorMovement( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put_CursorMovement( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_ControlCharacters( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ControlCharacters( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE _WSFunction( 
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT Arg16,
                 VARIANT Arg17,
                 VARIANT Arg18,
                 VARIANT Arg19,
                 VARIANT Arg20,
                 VARIANT Arg21,
                 VARIANT Arg22,
                 VARIANT Arg23,
                 VARIANT Arg24,
                 VARIANT Arg25,
                 VARIANT Arg26,
                 VARIANT Arg27,
                 VARIANT Arg28,
                 VARIANT Arg29,
                 VARIANT Arg30,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_EnableEvents( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableEvents( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayInfoWindow( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayInfoWindow( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Wait( 
                 VARIANT Time,
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_ExtendList( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ExtendList( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_OLEDBErrors( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE GetPhonetic( 
                 VARIANT Text,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_COMAddIns( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_DefaultWebOptions( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ProductCode( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_UserLibraryPath( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get_AutoPercentEntry( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_AutoPercentEntry( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_LanguageSettings( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Dummy101( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Dummy12( 
                 IDispatch *p1,
                 IDispatch *p2);

            HRESULT STDMETHODCALLTYPE get_AnswerWizard( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE CalculateFull( void);

            HRESULT STDMETHODCALLTYPE FindFile( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_CalculationVersion( 
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_ShowWindowsInTaskbar( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ShowWindowsInTaskbar( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_FeatureInstall( 
                 MsoFeatureInstall *RHS);

            HRESULT STDMETHODCALLTYPE put_FeatureInstall( 
                 MsoFeatureInstall RHS);

            HRESULT STDMETHODCALLTYPE get_Ready( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE Dummy13( 
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT Arg16,
                 VARIANT Arg17,
                 VARIANT Arg18,
                 VARIANT Arg19,
                 VARIANT Arg20,
                 VARIANT Arg21,
                 VARIANT Arg22,
                 VARIANT Arg23,
                 VARIANT Arg24,
                 VARIANT Arg25,
                 VARIANT Arg26,
                 VARIANT Arg27,
                 VARIANT Arg28,
                 VARIANT Arg29,
                 VARIANT Arg30,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_FindFormat( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE putref_FindFormat( 
                 IDispatch *RHS);

            HRESULT STDMETHODCALLTYPE get_ReplaceFormat( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE putref_ReplaceFormat( 
                 IDispatch *RHS);

            HRESULT STDMETHODCALLTYPE get_UsedObjects( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_CalculationState( 
                 XlCalculationState *RHS);

            HRESULT STDMETHODCALLTYPE get_CalculationInterruptKey( 
                 XlCalculationInterruptKey *RHS);

            HRESULT STDMETHODCALLTYPE put_CalculationInterruptKey( 
                 XlCalculationInterruptKey RHS);

            HRESULT STDMETHODCALLTYPE get_Watches( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayFunctionToolTips( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayFunctionToolTips( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_AutomationSecurity( 
                 MsoAutomationSecurity *RHS);

            HRESULT STDMETHODCALLTYPE put_AutomationSecurity( 
                 MsoAutomationSecurity RHS);

            HRESULT STDMETHODCALLTYPE get_FileDialog( 
                 MsoFileDialogType fileDialogType,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Dummy14( void);

            HRESULT STDMETHODCALLTYPE CalculateFullRebuild( void);

            HRESULT STDMETHODCALLTYPE get_DisplayPasteOptions( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayPasteOptions( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayInsertOptions( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayInsertOptions( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_GenerateGetPivotData( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_GenerateGetPivotData( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_AutoRecover( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Hwnd( 
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_Hinstance( 
                 long *RHS);

            HRESULT STDMETHODCALLTYPE CheckAbort( 
                 VARIANT KeepAbort);

            HRESULT STDMETHODCALLTYPE get_ErrorCheckingOptions( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_AutoFormatAsYouTypeReplaceHyperlinks( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_AutoFormatAsYouTypeReplaceHyperlinks( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_SmartTagRecognizers( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_NewWorkbook( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_SpellingOptions( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Speech( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_MapPaperSize( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_MapPaperSize( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_ShowStartupDialog( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_ShowStartupDialog( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_DecimalSeparator( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_DecimalSeparator( 
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_ThousandsSeparator( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_ThousandsSeparator( 
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_UseSystemSeparators( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_UseSystemSeparators( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_ThisCell( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_RTD( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayDocumentActionTaskPane( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayDocumentActionTaskPane( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE DisplayXMLSourcePane( 
                 VARIANT XmlMap);

            HRESULT STDMETHODCALLTYPE get_ArbitraryXMLSupportAvailable( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE Support( 
                 IDispatch *Object,
                 long ID,
                 VARIANT arg,
                 VARIANT *RHS);

		public:
			OpenOfficeWrapper::COOApplicationPtr GetOOApplication() const;
            HRESULT ResetWorkbooks();
			
		private:

            class SingletonSection
            {
            public:

                static SingletonSection* CreateInstance();
                static void FreeInstance();

                OpenOfficeWrapper::COOApplicationPtr GetOOApp();
                CWorkbooksPtr GetWorkbooks();


                void Initialize(CApplication* pApplication); 

            private:

                SingletonSection();
               
                ~SingletonSection();


                OpenOfficeWrapper::COOApplicationPtr m_pOOApp;    
                CWorkbooksPtr m_Workbooks;

                static int m_RefCounter;

                static SingletonSection* m_pInstance;

            };

            SingletonSection* m_pInstance;
            OpenOfficeWrapper::DispatchList m_garbageList;

    	};

        typedef CComObject<CApplication> CApplicationObject;
        typedef CComPtr<CApplication> CApplicationPtr;


        OBJECT_ENTRY_AUTO(__uuidof(Application), CApplication)
	}
}


#endif //_APPLICATION_H_