
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "StdAfx.h"
#include "Font.h"

using namespace AutomateIT;
using namespace AutomateIT::Utils;
using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::CalcItems;

CFont::CFont(): m_pRange(NULL)
{
    TRACINGMETHOD;
}

CFont::~CFont()
{
    TRACINGMETHOD;
}

HRESULT CFont::Initialize(CRangePtr pRange)
{
    TRACINGMETHOD;
    m_pRange = pRange;
    return hr;
}


HRESULT __stdcall CFont::get_Application( 
                                  _Application **RHS)
{
	TRACINGMETHOD;
	return m_pRange->get_Application(RHS);
}

HRESULT __stdcall CFont::get_Creator( 
                              XlCreator *RHS)
{
    TRACINGMETHOD;
    (*RHS) = xlCreatorCode;
    return hr;
}

HRESULT __stdcall CFont::get_Parent( 
                             IDispatch **RHS)
{
	TRACINGMETHOD;
    (*RHS) = m_pRange;
    (*RHS)->AddRef();
	return hr;
}

HRESULT __stdcall CFont::get_Background( 
                                 VARIANT *RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::put_Background( 
                                 VARIANT RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::get_Bold( 
                           VARIANT *pvbBold)
{ 
    TRACINGMETHOD; 
    // In OO bold is specified as weight of the character

    float fWeight(0);
    hr = m_pRange->GetRangeObject()->get_CharWeight(&fWeight);
    CHECKERROR;

    if (fWeight > 140)
    {
        (*pvbBold).boolVal = VARIANT_TRUE;
        (*pvbBold).vt = VT_BOOL;
    }
    else
    {
        (*pvbBold).boolVal = VARIANT_FALSE;
        (*pvbBold).vt = VT_BOOL;
    }
    return hr;
}

HRESULT __stdcall CFont::put_Bold( 
                           VARIANT vbBold)
{ 
    TRACINGMETHOD; 
    float fWeight(0);
    if (vbBold.boolVal == VARIANT_TRUE)
    {
        fWeight = 150;
    }
    else
    {
        fWeight = 100;
    }
    hr = m_pRange->GetRangeObject()->put_CharWeight(fWeight);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CFont::get_Color( 
                            VARIANT *RHS)
{ 
    TRACINGMETHOD; 
    int color(0);
    hr = m_pRange->GetRangeObject()->get_CharColor(&color);
    CHECKERROR;
    (*RHS) = CComVariant(CColorPalette::ConvertRGB(color));
    return hr; 
}

HRESULT __stdcall CFont::put_Color( 
                            VARIANT RHS)
{ 
    TRACINGMETHOD; 
    
    CComVariant color(RHS);
    hr = color.ChangeType(VT_I4);
    CHECKERROR;
    hr = m_pRange->GetRangeObject()->put_CharColor(CColorPalette::ConvertRGB(color.intVal));
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CFont::get_ColorIndex( 
                                 VARIANT *RHS)
{
    TRACINGMETHOD;
    int color(0);
    hr = m_pRange->GetRangeObject()->get_CharColor(&color);
    CHECKERROR;
    (*RHS) = CComVariant(CColorPalette::GetColorIndex(color) + 1);
    return hr;
}

HRESULT __stdcall CFont::put_ColorIndex( 
                                 VARIANT RHS)
{
    TRACINGMETHOD; 
    CComVariant colorIndex(RHS);
    hr = colorIndex.ChangeType(VT_I4);
    CHECKERROR;
    if (colorIndex.intVal == xlColorIndexNone) 
    {
        colorIndex.intVal = 2;
    }
    if (colorIndex.intVal == xlColorIndexAutomatic) 
    {
        colorIndex.intVal = 1;
    }
    if (colorIndex.intVal == 0) 
    {
        colorIndex.intVal = 2;
    }

    hr = m_pRange->GetRangeObject()->put_CharColor(CColorPalette::GetColor(colorIndex.intVal - 1));
    return hr;
}

HRESULT __stdcall CFont::get_FontStyle( 
                                VARIANT *RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::put_FontStyle( 
                                VARIANT RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::get_Italic( 
                             VARIANT *pvbItalic)
{ 
    TRACINGMETHOD; 

    int posture(0);
    hr = m_pRange->GetRangeObject()->get_CharPosture(&posture);
    CHECKERROR;
    if (posture != 0)
    {
        (*pvbItalic).boolVal = VARIANT_TRUE;
        (*pvbItalic).vt = VT_BOOL;
    }
    else
    {
        (*pvbItalic).boolVal = VARIANT_FALSE;
        (*pvbItalic).vt = VT_BOOL;
    }
    return hr; 
}

HRESULT __stdcall CFont::put_Italic( 
                             VARIANT vbItalic)
{ 
    TRACINGMETHOD; 
    int posture;
    if (vbItalic.boolVal == VARIANT_TRUE)
    {
        posture = 3;
    }
    else
    {
        posture = 0;
    }
    hr = m_pRange->GetRangeObject()->put_CharPosture(posture);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CFont::get_Name( 
                           VARIANT *RHS)
{
	TRACINGMETHOD;

    BSTR fontName;
    hr = m_pRange->GetRangeObject()->get_CharFontName(&fontName);
    CHECKERROR;

    *RHS = CComVariant(fontName);
	return hr;
}

HRESULT __stdcall CFont::put_Name( 
                           VARIANT RHS)
{
	TRACINGMETHOD;
    CComVariant fontName(RHS);
    hr = fontName.ChangeType(VT_BSTR);
    CHECKERROR;

    hr = m_pRange->GetRangeObject()->put_CharFontName(fontName.bstrVal);
    CHECKERROR;
    
    return hr;
}

HRESULT __stdcall CFont::get_OutlineFont( 
                                  VARIANT *RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::put_OutlineFont( 
                                  VARIANT RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::get_Shadow( 
                             VARIANT *RHS)
{
    TRACINGMETHOD; 
    bool shadow(false);
    hr = m_pRange->GetRangeObject()->get_CharShadowed(&shadow);
    CHECKERROR;
    CComVariant vShadow(shadow);
    (*RHS) = vShadow;
    return hr;
}

HRESULT __stdcall CFont::put_Shadow( 
                             VARIANT RHS)
{
    TRACINGMETHOD; 
    CComVariant shadow(RHS);
    hr = shadow.ChangeType(VT_BOOL);
    CHECKERROR;
    hr = m_pRange->GetRangeObject()->put_CharShadowed(shadow.boolVal != VARIANT_FALSE);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CFont::get_Size( 
                           VARIANT *RHS)
{ 
    TRACINGMETHOD; 
    int size(0);
    hr = m_pRange->GetRangeObject()->get_CharHeight(&size);
    CHECKERROR;
    CComVariant vSize(size);
    (*RHS) = vSize;
    return hr;
}

HRESULT __stdcall CFont::put_Size( 
                           VARIANT RHS)
{ 
    TRACINGMETHOD; 
    CComVariant size(RHS);
    hr = size.ChangeType(VT_I4);
    CHECKERROR;
    hr = m_pRange->GetRangeObject()->put_CharHeight(size.intVal);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CFont::get_Strikethrough( 
                                    VARIANT *RHS)
{
    TRACINGMETHOD; 
    bool strikethrough(false);
    hr = m_pRange->GetRangeObject()->get_CharStrikeout(&strikethrough);
    CHECKERROR;
    CComVariant vStrikethrough(strikethrough);
    (*RHS) = vStrikethrough;
    return hr;
}

HRESULT __stdcall CFont::put_Strikethrough( 
                                    VARIANT RHS)
{
    TRACINGMETHOD; 
    CComVariant strikethrough(RHS);
    hr = strikethrough.ChangeType(VT_BOOL);
    CHECKERROR;
    hr = m_pRange->GetRangeObject()->put_CharStrikeout(strikethrough.boolVal != VARIANT_FALSE);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CFont::get_Subscript( 
                                VARIANT *RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::put_Subscript( 
                                VARIANT RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::get_Superscript( 
                                  VARIANT *RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CFont::put_Superscript( 
                                  VARIANT RHS)
{
	TRACINGMETHOD;
	return S_OK;
}

HRESULT __stdcall CFont::get_Underline( 
                                VARIANT *pvbUnderline)
{ 
    TRACINGMETHOD; 
    bool underline(false);
    hr = m_pRange->GetRangeObject()->get_CharUnderline(&underline);
    CHECKERROR;
    CComVariant vUnderline(underline ? xlUnderlineStyleSingle : xlUnderlineStyleNone);
    (*pvbUnderline) = vUnderline;
    return hr;
}

HRESULT __stdcall CFont::put_Underline( 
                                VARIANT vbUnderline)
{ 
    TRACINGMETHOD; 
    CComVariant vUnderline(vbUnderline);
    hr = vUnderline.ChangeType(VT_INT);
    CHECKERROR;
    hr = m_pRange->GetRangeObject()->put_CharUnderline(vUnderline.intVal != xlUnderlineStyleNone);
    CHECKERROR;
    return hr;
}