/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "PageSetup.h"

using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::OpenOfficeWrapper;

CPageSetup::CPageSetup() : 
    m_pSheet(NULL),
    m_pPageSetupStyle(NULL)
{
    TRACINGMETHOD;
}

CPageSetup::~CPageSetup()
{
    TRACINGMETHOD;
}

HRESULT __stdcall CPageSetup::Initialize(CRange* pSheet, COOPageSetupStylePtr pStyle)
{
    TRACINGMETHOD;
    m_pSheet = pSheet;
    m_pPageSetupStyle = pStyle;
    return hr;
}

HRESULT __stdcall CPageSetup::get_Application( 
                                  _Application **RHS) 
{ 
    TRACINGMETHOD;
    return m_pSheet->get_Application(RHS);
}

HRESULT __stdcall CPageSetup::get_Creator( 
    XlCreator *RHS) 
{ 
    TRACINGMETHOD;
    (*RHS) = xlCreatorCode;
    return hr;
}

HRESULT __stdcall CPageSetup::get_Parent( 
    IDispatch **RHS) 
{ 
    TRACINGMETHOD;
    (*RHS) = m_pSheet;
    (*RHS)->AddRef();
    return hr;
}

HRESULT __stdcall CPageSetup::get_BlackAndWhite( 
    VARIANT_BOOL *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_BlackAndWhite( 
    VARIANT_BOOL RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_BottomMargin( 
    double *RHS)  
{ 
    TRACINGMETHOD; 
    int margin(0);
    hr = m_pPageSetupStyle->get_BottomMargin(&margin);
    CHECKERROR;
    (*RHS) = margin;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_BottomMargin( 
    double RHS)  
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_BottomMargin(static_cast<int>(RHS));
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_CenterFooter( 
    BSTR *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_CenterFooter( 
    BSTR RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_CenterHeader( 
    BSTR *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_CenterHeader( 
    BSTR RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_CenterHorizontally( 
    VARIANT_BOOL *RHS)  
{ 
    TRACINGMETHOD; 
    bool centerHorizontally(false);
    hr = m_pPageSetupStyle->get_CenterHorizontally(&centerHorizontally);
    CHECKERROR;
    (*RHS) = centerHorizontally ? VARIANT_TRUE : VARIANT_FALSE;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_CenterHorizontally( 
    VARIANT_BOOL RHS)  
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_CenterHorizontally(RHS != VARIANT_FALSE);
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_CenterVertically( 
    VARIANT_BOOL *RHS)  
{ 
    TRACINGMETHOD; 
    bool centerVertically(false);
    hr = m_pPageSetupStyle->get_CenterVertically(&centerVertically);
    CHECKERROR;
    (*RHS) = centerVertically ? VARIANT_TRUE : VARIANT_FALSE;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_CenterVertically( 
    VARIANT_BOOL RHS)  
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_CenterVertically(RHS != VARIANT_FALSE);
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_ChartSize( 
    XlObjectSize *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_ChartSize( 
    XlObjectSize RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_Draft( 
                                        VARIANT_BOOL *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_Draft( 
                                        VARIANT_BOOL RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_FirstPageNumber( 
    long *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_FirstPageNumber( 
    long RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_FitToPagesTall( 
    VARIANT *RHS)  
{ 
    TRACINGMETHOD; 
    short scaleToPagesY(0);
    hr = m_pPageSetupStyle->get_ScaleToPagesY(&scaleToPagesY);
    CHECKERROR;
    (*RHS) = CComVariant(scaleToPagesY);
    return hr; 
}

HRESULT __stdcall CPageSetup::put_FitToPagesTall( 
    VARIANT RHS)  
{ 
    TRACINGMETHOD; 
    CComVariant fitToPagesTall(RHS);
    hr = fitToPagesTall.ChangeType(VT_I2);
    CHECKERROR;
    hr = m_pPageSetupStyle->put_ScaleToPagesY(fitToPagesTall.iVal);
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_FitToPagesWide( 
    VARIANT *RHS)  
{ 
    TRACINGMETHOD; 
    short scaleToPagesX(0);
    hr = m_pPageSetupStyle->get_ScaleToPagesX(&scaleToPagesX);
    CHECKERROR;
    (*RHS) = CComVariant(scaleToPagesX);
    return hr; 
}

HRESULT __stdcall CPageSetup::put_FitToPagesWide( 
    VARIANT RHS) 
{ 
    TRACINGMETHOD; 
    CComVariant fitToPagesWide(RHS);
    hr = fitToPagesWide.ChangeType(VT_I2);
    CHECKERROR;
    hr = m_pPageSetupStyle->put_ScaleToPagesX(fitToPagesWide.iVal);
    CHECKERROR;
    return hr;  
}

HRESULT __stdcall CPageSetup::get_FooterMargin( 
    double *RHS)  
{ 
    TRACINGMETHOD; 
    long footerMargin(0);
    hr = m_pPageSetupStyle->get_FooterHeight(&footerMargin);
    CHECKERROR;
    (*RHS) = footerMargin;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_FooterMargin( 
    double RHS)  
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_FooterHeight(static_cast<long>(RHS));
    CHECKERROR;
    return hr;  
}

HRESULT __stdcall CPageSetup::get_HeaderMargin( 
    double *RHS)  
{ 
    TRACINGMETHOD; 
    long headerMargin(0);
    hr = m_pPageSetupStyle->get_HeaderHeight(&headerMargin);
    CHECKERROR;
    (*RHS) = headerMargin;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_HeaderMargin( 
    double RHS)  
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_HeaderHeight(static_cast<long>(RHS));
    CHECKERROR;
    return hr;  
}

HRESULT __stdcall CPageSetup::get_LeftFooter( 
    BSTR *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_LeftFooter( 
    BSTR RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_LeftHeader( 
    BSTR *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_LeftHeader( 
    BSTR RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_LeftMargin( 
    double *RHS)  
{ 
    TRACINGMETHOD; 
    int margin(0);
    hr = m_pPageSetupStyle->get_LeftMargin(&margin);
    CHECKERROR;
    (*RHS) = margin;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_LeftMargin( 
    double RHS) 
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_LeftMargin(static_cast<int>(RHS));
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_Order( 
                                        XlOrder *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_Order( 
                                        XlOrder RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_Orientation( 
    XlPageOrientation *RHS) 
{ 
    TRACINGMETHOD; 
    bool isLangscape(false);
    hr = m_pPageSetupStyle->get_IsLandscape(&isLangscape);
    CHECKERROR;
    (*RHS) = isLangscape ? xlLandscape : xlPortrait;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_Orientation( 
    XlPageOrientation RHS) 
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_IsLandscape(RHS == xlLandscape);
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_PaperSize( 
    XlPaperSize *RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PaperSize( 
    XlPaperSize RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintArea( 
    BSTR *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintArea( 
    BSTR RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintGridlines( 
    VARIANT_BOOL *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintGridlines( 
    VARIANT_BOOL RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintHeadings( 
    VARIANT_BOOL *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintHeadings( 
    VARIANT_BOOL RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintNotes( 
    VARIANT_BOOL *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintNotes( 
    VARIANT_BOOL RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintQuality( 
                                   VARIANT Index,
                                   VARIANT *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintQuality( 
                                   VARIANT Index,
                                   VARIANT RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintTitleColumns( 
    BSTR *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintTitleColumns( 
    BSTR RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintTitleRows( 
    BSTR *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    (*RHS) = ::SysAllocString(OLESTR(""));
    // stub
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintTitleRows( 
    BSTR RHS)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    // stub
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_RightFooter( 
    BSTR *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_RightFooter( 
    BSTR RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_RightHeader( 
    BSTR *RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_RightHeader( 
    BSTR RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_RightMargin( 
    double *RHS) 
{ 
    TRACINGMETHOD; 
    int margin(0);
    hr = m_pPageSetupStyle->get_RightMargin(&margin);
    CHECKERROR;
    (*RHS) = margin;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_RightMargin( 
    double RHS) 
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_RightMargin(static_cast<int>(RHS));
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_TopMargin( 
    double *RHS) 
{ 
    TRACINGMETHOD; 
    int margin(0);
    hr = m_pPageSetupStyle->get_TopMargin(&margin);
    CHECKERROR;
    (*RHS) = margin;
    return hr; 
}

HRESULT __stdcall CPageSetup::put_TopMargin( 
    double RHS) 
{ 
    TRACINGMETHOD; 
    hr = m_pPageSetupStyle->put_TopMargin(static_cast<int>(RHS));
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_Zoom( 
                                       VARIANT *RHS) 
{ 
    TRACINGMETHOD; 
    short zoom(0);
    CHECKERROR;
    hr = m_pPageSetupStyle->get_PageScale(&zoom);
    CHECKERROR;
    (*RHS) = CComVariant(zoom);
    return hr; 
}

HRESULT __stdcall CPageSetup::put_Zoom( 
                                       VARIANT RHS) 
{ 
    TRACINGMETHOD; 
    CComVariant zoom(RHS);
    hr = zoom.ChangeType(VT_I2);
    CHECKERROR;
    hr = m_pPageSetupStyle->put_PageScale(zoom.iVal);
    CHECKERROR;
    return hr; 
}

HRESULT __stdcall CPageSetup::get_PrintComments( 
    XlPrintLocation *RHS)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintComments( 
    XlPrintLocation RHS)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_PrintErrors( 
    XlPrintErrors *RHS)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::put_PrintErrors( 
    XlPrintErrors RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_CenterHeaderPicture( 
    IDispatch **RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_CenterFooterPicture( 
    IDispatch **RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_LeftHeaderPicture( 
    IDispatch **RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_LeftFooterPicture( 
    IDispatch **RHS)  
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_RightHeaderPicture( 
    IDispatch **RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}

HRESULT __stdcall CPageSetup::get_RightFooterPicture( 
    IDispatch **RHS) 
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL; 
    return S_OK; 
}