/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "Range.h"

using namespace AutomateIT;
using namespace AutomateIT::Utils;
using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::CalcItems;
using namespace AutomateIT::OpenOfficeWrapper;

CRange::CRange() : 
m_pRange(NULL), 
m_ParentApplication(NULL), 
m_pSheet(NULL), 
m_pBorders(NULL),
m_InternalType(xlColumns)
{
    TRACINGMETHOD;
}

CRange::~CRange()
{
    TRACINGMETHOD;
}

STDMETHODIMP CRange::Initialize(CApplication* application, CWorksheet* sheet, COORangePtr pOORange) 
{
    TRACINGMETHOD;
    m_pRange = pOORange;
    m_ParentApplication = application;
    m_pSheet = sheet;
    return hr;
}

STDMETHODIMP CRange::Initialize(CApplication* application, CWorksheet* sheet, COORangePtr pOORange, const CPoint& topLeft, const CPoint& bottomRight)
{
    TRACINGMETHOD;

    COORangePtr pRange(NULL);
    hr = pOORange->GetCellRangeByPosition(topLeft.x - 1, topLeft.y - 1, bottomRight.x - 1, bottomRight.y - 1, &pRange);
    CHECKERROR;

    m_pRange = pRange;
    m_ParentApplication = application;
    m_pSheet = sheet;

    return hr;
}


STDMETHODIMP CRange::Initialize(CApplication* application, CWorksheet* sheet, COORangePtr pOORange, const std::wstring& name)
{
    TRACINGMETHOD;

    CRangeAddress address(name);
    CComVariant vName = address.ToString().c_str();
    COORangePtr pRange(NULL);
    hr = pOORange->GetCellRangeByName(vName.bstrVal, &pRange);
    CHECKERROR;

    m_pRange = pRange;
    m_ParentApplication = application;
    m_pSheet = sheet;
    return hr;
}


HRESULT CRange::GetRangeAddress(CRangeAddress& address)
{
    TRACINGMETHOD;
    COORangeAddressPtr pRangeAddress(NULL);
    hr = m_pRange->GetRangeAddress(&pRangeAddress);
    CHECKERROR;

    long lTop(0);
    hr = pRangeAddress->get_StartRow(&lTop);
    CHECKERROR;

    long lLeft(0);
    hr = pRangeAddress->get_StartColumn(&lLeft);
    CHECKERROR;

    long lBottom(0);
    hr = pRangeAddress->get_EndRow(&lBottom);
    CHECKERROR;

    long lRight(0);
    hr = pRangeAddress->get_EndColumn(&lRight);
    CHECKERROR;

    address = CRangeAddress(CPoint(lLeft + 1, lTop + 1), CPoint(lRight + 1, lBottom + 1));
    return hr;
}


COORangePtr CRange::GetRangeObject() const
{
    return m_pRange;
}

void CRange::SetInternalType(XlRowCol internalType)
{
    m_InternalType = internalType;
}

HRESULT __stdcall CRange::get_Application( 
    _Application **RHS)
{
    TRACINGMETHOD;
    (*RHS) = m_ParentApplication;
    (*RHS)->AddRef();
    return hr;
}

HRESULT __stdcall CRange::get_Creator( 
                                      XlCreator *RHS)
{
    TRACINGMETHOD;
    (*RHS) = xlCreatorCode;
    return hr;
}

HRESULT __stdcall CRange::get_Parent( 
                                     IDispatch **RHS)
{
    TRACINGMETHOD;
    (*RHS) = m_pSheet;
    (*RHS)->AddRef();
    return hr;
}

HRESULT __stdcall CRange::Activate( 
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_AddIndent( 
                                        VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_AddIndent( 
                                        VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Address( 
                                      VARIANT RowAbsolute,
                                      VARIANT ColumnAbsolute,
                                      XlReferenceStyle ReferenceStyle,
                                      VARIANT External,
                                      VARIANT RelativeTo,
                                      long lcid,
                                      BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_AddressLocal( 
    VARIANT RowAbsolute,
    VARIANT ColumnAbsolute,
    XlReferenceStyle ReferenceStyle,
    VARIANT External,
    VARIANT RelativeTo,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::AdvancedFilter( 
    XlFilterAction Action,
    VARIANT CriteriaRange,
    VARIANT CopyToRange,
    VARIANT Unique,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::ApplyNames( 
                                     VARIANT Names,
                                     VARIANT IgnoreRelativeAbsolute,
                                     VARIANT UseRowColumnNames,
                                     VARIANT OmitColumn,
                                     VARIANT OmitRow,
                                     XlApplyNamesOrder Order,
                                     VARIANT AppendLast,
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::ApplyOutlineStyles( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Areas( 
                                    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::AutoComplete( 
                                       BSTR String,
                                       BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::AutoFill( 
                                   Range *Destination,
                                   XlAutoFillType Type,
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::AutoFilter( 
                                     VARIANT Field,
                                     VARIANT Criteria1,
                                     XlAutoFilterOperator Operator,
                                     VARIANT Criteria2,
                                     VARIANT VisibleDropDown,
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::AutoFit( 
                                  VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;

}

HRESULT __stdcall CRange::AutoFormat( 
                                     XlRangeAutoFormat Format,
                                     VARIANT Number,
                                     VARIANT IFont,
                                     VARIANT Alignment,
                                     VARIANT Border,
                                     VARIANT Pattern,
                                     VARIANT Width,
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::AutoOutline( 
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::BorderAround( 
                                       VARIANT LineStyle,
                                       XlBorderWeight Weight,
                                       XlColorIndex ColorIndex,
                                       VARIANT Color,
                                       VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Borders( 
                                      Borders **RHS)
{
    TRACINGMETHOD;
    if (m_pBorders.p == NULL)
    {
        CBordersObject* pBordersObject(NULL);
        hr = CBordersObject::CreateInstance(&pBordersObject);
        CHECKERROR;

        m_pBorders = (pBordersObject);
        hr = m_pBorders->Initialize(this, m_pRange);
        CHECKERROR;
    }

    (*RHS) = m_pBorders;
    (*RHS)->AddRef();

    return hr;
}

HRESULT __stdcall CRange::Calculate( 
                                    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Cells( 
                                    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Characters( 
    VARIANT Start,
    VARIANT Length,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::CheckSpelling( 
                                        VARIANT CustomDictionary,
                                        VARIANT IgnoreUppercase,
                                        VARIANT AlwaysSuggest,
                                        VARIANT SpellLang,
                                        VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Clear( 
                                VARIANT *RHS)
{
    TRACINGMETHOD;

    hr = m_pRange->ClearContents(
        COORange::VALUE | 
        COORange::DATETIME | 
        COORange::STRING | 
        COORange::ANNOTATION | 
        COORange::FORMULA | 
        COORange::HARDATTR | 
        COORange::STYLES | 
        COORange::OBJECTS | 
        COORange::EDITATTR | 
        COORange::FORMATTED);

    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::ClearContents( 
                                        VARIANT *RHS)
{
    TRACINGMETHOD;

    hr = m_pRange->ClearContents(
        COORange::VALUE | 
        COORange::DATETIME | 
        COORange::STRING | 
        COORange::FORMULA | 
        COORange::OBJECTS);

    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::ClearFormats( 
                                       VARIANT *RHS)
{
    TRACINGMETHOD;

    hr = m_pRange->ClearContents(
        COORange::HARDATTR | 
        COORange::STYLES | 
        COORange::EDITATTR | 
        COORange::FORMATTED);

    CHECKERROR;

    return hr;

}

HRESULT __stdcall CRange::ClearNotes( 
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    return ClearComments();
}

HRESULT __stdcall CRange::ClearOutline( 
                                       VARIANT *RHS)
{
    TRACINGMETHOD;

    hr = m_pRange->ClearContents(COORange::STYLES);

    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::get_Column( 
                                     long *RHS)
{ 
    TRACINGMETHOD; 
    COORangeAddressPtr pRangeAddress(NULL);
    hr = m_pRange->GetRangeAddress(&pRangeAddress);
    CHECKERROR;

    long startCol(0);
    hr = pRangeAddress->get_StartColumn(&startCol);
    CHECKERROR;

    *RHS = startCol + 1;
    return hr;
}

HRESULT __stdcall CRange::ColumnDifferences( 
    VARIANT Comparison,
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Columns( 
                                      Range **RHS)
{
    TRACINGMETHOD; 

    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;
    CRangeObject* pRange(NULL);
    hr = CRangeObject::CreateInstance(&pRange);
    CHECKERROR;
    CRangePtr pRangePtr(pRange);

    COORangePtr sheetPtr(NULL);
    hr = m_pRange->get_Spreadsheet(&sheetPtr);
    CHECKERROR;

    hr = pRangePtr->Initialize(m_ParentApplication, m_pSheet, sheetPtr, CPoint(address.GetLeft() + 1, CAddressItem::MinRow() + 1), CPoint(address.GetRight() + 1, CAddressItem::MaxRow() + 1));
    CHECKERROR;
    (*RHS) = pRangePtr;
    (*RHS)->AddRef();
    return hr;
}

HRESULT __stdcall CRange::get_ColumnWidth( 
    VARIANT *RHS)
{ 
    TRACINGMETHOD; 

    COOColumnsPtr pColumns(NULL); 
    hr = m_pRange->GetColumns(&pColumns);
    CHECKERROR;

    int width(0);
    hr = pColumns->get_Width(&width);
    CHECKERROR;

    CComVariant vWidth(width / 200);
    (*RHS) = vWidth;

    return hr;
}

HRESULT __stdcall CRange::put_ColumnWidth( 
    VARIANT RHS)
{ 
    TRACINGMETHOD; 

    COOColumnsPtr pColumns(NULL); 
    hr = m_pRange->GetColumns(&pColumns);
    CHECKERROR;

    CComVariant vWidth(RHS);
    hr = vWidth.ChangeType(VT_I4);
    CHECKERROR;

    hr = pColumns->put_Width(vWidth.intVal * 210);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::Consolidate( 
                                      VARIANT Sources,
                                      VARIANT Function,
                                      VARIANT TopRow,
                                      VARIANT LeftColumn,
                                      VARIANT CreateLinks,
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Copy( 
                               VARIANT Destination,
                               VARIANT *RHS)
{
    TRACINGMETHOD;

    CComVariant selected;

    hr = Select(&selected);
    CHECKERROR;

    COODispatchHelperPtr pDispatchHelper(NULL);
    hr = m_ParentApplication->GetOOApplication()->CreateDispatchHelper(&pDispatchHelper);

    IDispatchPtr pFrame(NULL);
    hr = m_pSheet->GetParentWorkbook()->GetOODocument()->GetDocument()->GetFrame(&pFrame);
    CHECKERROR;


    hr = pDispatchHelper->ExecuteDispatch(pFrame, OLESTR(".uno:Copy"));
    CHECKERROR;

    if (Destination.vt == VT_DISPATCH) 
    {
        IDispatchPtr pDisp = Destination.pdispVal;
        CComPtr<Range> pDestRange(NULL);
        hr = pDisp->QueryInterface(&pDestRange);
        CHECKERROR;

        pDestRange->Select(&selected);

        CRange* pRangeImpl = reinterpret_cast<CRange*>(pDestRange.p);

        IDispatchPtr pFrame2(NULL);
        hr = pRangeImpl->m_pSheet->GetParentWorkbook()->GetOODocument()->GetDocument()->GetFrame(&pFrame2);
        CHECKERROR;

        hr = pDispatchHelper->ExecuteDispatch(pFrame2, OLESTR(".uno:Paste"));
        CHECKERROR;

        IDispatch* res = pRangeImpl;
        (*RHS) = CComVariant(res);
        res->AddRef();
    }
    else
    {
        IDispatch* res = this;
        (*RHS) = CComVariant(res);
        res->AddRef();
    }


    return hr;
}

HRESULT __stdcall CRange::CopyFromRecordset( 
    IUnknown *Data,
    VARIANT MaxRows,
    VARIANT MaxColumns,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::CopyPicture( 
                                      XlPictureAppearance Appearance,
                                      XlCopyPictureFormat Format,
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Count( 
                                    long *RHS)
{
    TRACINGMETHOD;
    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;
    (*RHS) = address.GetCount();

    return hr;
}

HRESULT __stdcall CRange::CreateNames( 
                                      VARIANT Top,
                                      VARIANT Left,
                                      VARIANT Bottom,
                                      VARIANT Right,
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::CreatePublisher( 
    VARIANT Edition,
    XlPictureAppearance Appearance,
    VARIANT ContainsPICT,
    VARIANT ContainsBIFF,
    VARIANT ContainsRTF,
    VARIANT ContainsVALU,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_CurrentArray( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_CurrentRegion( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Cut( 
                              VARIANT Destination,
                              VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::DataSeries( 
                                     VARIANT Rowcol,
                                     XlDataSeriesType Type,
                                     XlDataSeriesDate Date,
                                     VARIANT Step,
                                     VARIANT Stop,
                                     VARIANT Trend,
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get__Default( 
                                       VARIANT RowIndex,
                                       VARIANT ColumnIndex,
                                       long lcid,
                                       VARIANT *RHS)
{ 
    TRACINGMETHOD; 

    CRangeObject* pCell(NULL);
    hr = CRangeObject::CreateInstance(&pCell);
    CHECKERROR;
    CRangePtr pCellPtr(pCell);

    CComVariant row(RowIndex);
    CComVariant column(ColumnIndex);

    if (row.vt != VT_BSTR)
    {

        CPoint topLeft(0, 0);
        CPoint bottomRight(0, 0);
        hr = row.ChangeType(VT_I4);
        CHECKERROR;

        CRangeAddress address(OLESTR("A1:A1"));
        hr = GetRangeAddress(address);
        CHECKERROR;

        if (FAILED(column.ChangeType(VT_I4)))
        {
            if (address.IsColumns() && address.IsRows())
            {
                if (m_InternalType == xlColumns)
                {
                    topLeft.x = row.intVal + address.GetLeft();
                    topLeft.y = CAddressItem::MinRow() + 1;
                    bottomRight.x = row.intVal + address.GetLeft();
                    bottomRight.y = CAddressItem::MaxRow() + 1;
                }
                if (m_InternalType == xlRows)
                {
                    topLeft.x = CAddressItem::MinCol() + 1;
                    topLeft.y = row.intVal + address.GetTop();
                    bottomRight.x = CAddressItem::MaxCol() + 1;
                    bottomRight.y = row.intVal + address.GetTop();
                }
            }
            else
            {
                if (address.IsColumns())
                {
                    topLeft.x = row.intVal + address.GetLeft();
                    topLeft.y = CAddressItem::MinRow() + 1;
                    bottomRight.x = row.intVal + address.GetLeft();
                    bottomRight.y = CAddressItem::MaxRow() + 1;
                }
                else if (address.IsRows())
                {
                    topLeft.x = CAddressItem::MinCol() + 1;
                    topLeft.y = row.intVal + address.GetTop();
                    bottomRight.x = CAddressItem::MaxCol() + 1;
                    bottomRight.y = row.intVal + address.GetTop();
                }
                else 
                {
                    hr = E_FAIL;
                    CHECKERROR;
                }
            }
        }
        else
        {
            topLeft.x = column.intVal + address.GetLeft();
            topLeft.y = row.intVal + address.GetTop();
            bottomRight.x = column.intVal + address.GetLeft();
            bottomRight.y = row.intVal + address.GetTop();
        }
        // getting parent sheet
        COORangePtr pSpreadsheetPtr(NULL);
        hr = m_pRange->get_Spreadsheet(&pSpreadsheetPtr);
        CHECKERROR;
        hr = pCellPtr->Initialize(m_ParentApplication, m_pSheet, pSpreadsheetPtr, topLeft, bottomRight);
    }
    else
    {
        COORangePtr pSpreadsheetPtr(NULL);
        hr = m_pRange->get_Spreadsheet(&pSpreadsheetPtr);
        CHECKERROR;
        hr = pCellPtr->Initialize(m_ParentApplication, m_pSheet, pSpreadsheetPtr, std::wstring(row.bstrVal));
    }

    CHECKERROR;

    (*RHS).vt = VT_DISPATCH;
    (*RHS).pdispVal = pCellPtr.p;
    pCell->AddRef();
    return hr;
}

HRESULT __stdcall CRange::put__Default( 
                                       VARIANT RowIndex,
                                       VARIANT ColumnIndex,
                                       long lcid,
                                       VARIANT RHS)
{
    TRACINGMETHOD; 

    CComVariant row(RowIndex);
    CComVariant column(ColumnIndex);

    hr = row.ChangeType(VT_I4);
    CHECKERROR;
    hr = column.ChangeType(VT_I4);
    CHECKERROR;
    COOCellPtr pCell(NULL);
    hr = m_pRange->GetCellByPosition(column.intVal - 1, row.intVal - 1, &pCell);
    CHECKERROR;
    hr = pCell->put_FormulaLocal(RHS);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::Delete( 
                                 VARIANT Shift,
                                 VARIANT *RHS)
{
    TRACINGMETHOD;
    XlDeleteShiftDirection action(xlShiftToLeft);

    CComVariant vShift(Shift);

    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;

    if (vShift.vt == VT_NULL || vShift.vt == VT_EMPTY || vShift.vt == VT_ERROR )
    {
        if ((address.GetRight() - address.GetLeft()) > (address.GetBottom() - address.GetTop()))
        {
            action = xlShiftUp;
        }
    }
    else
    {
        hr = vShift.ChangeType(VT_I4);
        CHECKERROR;
        if (vShift.intVal != xlShiftToLeft && vShift.intVal != xlShiftUp)
        {
            action = xlShiftToLeft;
        }
        else
        {
            action = (XlDeleteShiftDirection)vShift.intVal;
        }

    }

    switch(action)
    {
    case xlShiftToLeft:
        {
            COOColumnsPtr pColumns(NULL);
            hr = m_pRange->GetColumns(&pColumns);
            CHECKERROR;
            hr = pColumns->RemoveByIndex(0, address.GetRight() - address.GetLeft() + 1);
            CHECKERROR;
            break;
        }
    case xlShiftUp:
        {
            COORowsPtr pRows(NULL);
            hr = m_pRange->GetRows(&pRows);
            CHECKERROR;
            hr = pRows->RemoveByIndex(0, address.GetBottom() - address.GetTop() + 1);
            CHECKERROR;
            break;
        }
    }

    IDispatch* result = this;
    (*RHS) = CComVariant(result);
    (result)->AddRef();

    return hr;
}

HRESULT __stdcall CRange::get_Dependents( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::DialogBox1( 
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_DirectDependents( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_DirectPrecedents( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::EditionOptions( 
    XlEditionType Type,
    XlEditionOptionsOption Option,
    VARIANT Name,
    VARIANT Reference,
    XlPictureAppearance Appearance,
    XlPictureAppearance ChartSize,
    VARIANT Format,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_End( 
                                  XlDirection Direction,
                                  Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_EntireColumn( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_EntireRow( 
                                        Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::FillDown( 
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::FillLeft( 
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::FillRight( 
                                    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::FillUp( 
                                 VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Find( 
                               VARIANT What,
                               VARIANT After,
                               VARIANT LookIn,
                               VARIANT LookAt,
                               VARIANT SearchOrder,
                               XlSearchDirection SearchDirection,
                               VARIANT MatchCase,
                               VARIANT MatchByte,
                               VARIANT SearchFormat,
                               Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::FindNext( 
                                   VARIANT After,
                                   Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::FindPrevious( 
                                       VARIANT After,
                                       Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Font( 
                                   Font **RHS)
{ 
    TRACINGMETHOD; 
    if (RHS == NULL)
    {
        return E_POINTER;
    }
    *RHS = NULL;
    try
    {
        CFontObject* pFont = new CFontObject(this);
        HRESULT hr = pFont->Initialize(this);
        CHECKERROR;

        pFont->AddRef();
        *RHS = pFont;
        return hr;
    }
    catch (std::bad_alloc&)
    {
        return E_OUTOFMEMORY;
    }
}

HRESULT __stdcall CRange::get_Formula( 
                                      long lcid,
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_Formula( 
                                      long lcid,
                                      VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_FormulaArray( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_FormulaArray( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_FormulaLabel( 
    XlFormulaLabel *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_FormulaLabel( 
    XlFormulaLabel RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_FormulaHidden( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_FormulaHidden( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_FormulaLocal( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_FormulaLocal( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_FormulaR1C1( 
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_FormulaR1C1( 
    long lcid,
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_FormulaR1C1Local( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_FormulaR1C1Local( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::FunctionWizard( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::GoalSeek( 
                                   VARIANT Goal,
                                   Range *ChangingCell,
                                   VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Group( 
                                VARIANT Start,
                                VARIANT End,
                                VARIANT By,
                                VARIANT Periods,
                                VARIANT *RHS)
{ 
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_HasArray( 
                                       VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_HasFormula( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Height( 
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    short height(0);
    hr = m_pRange->get_Height(&height);
    CHECKERROR;

    (*RHS) = CComVariant((height + 1) / 100);

    return hr;
}

HRESULT __stdcall CRange::get_Hidden( 
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    bool hidden(false);
    hr = m_pRange->get_IsHidden(&hidden);
    CHECKERROR;

    (*RHS) = CComVariant(hidden);

    return hr;
}


HRESULT __stdcall CRange::put_Hidden( 
                                     VARIANT RHS)
{
    TRACINGMETHOD;

    CComVariant hidden(RHS);
    hr = hidden.ChangeType(VT_BOOL);
    CHECKERROR;

    hr = m_pRange->put_IsHidden(hidden.boolVal != VARIANT_FALSE);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::get_HorizontalAlignment( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    int justify(0);
    hr = m_pRange->get_HoriJustify(&justify);
    CHECKERROR;

    CComVariant align(0);
    switch (justify)
    {
    case 0:
        align = CComVariant(xlHAlignDistributed);
        break;
    case 1:
        align = CComVariant(xlHAlignLeft);
        break;
    case 2:
        align = CComVariant(xlHAlignCenter);
        break;
    case 3:
        align = CComVariant(xlHAlignRight);
        break;
    case 4:
        align = CComVariant(xlHAlignJustify);
        break;
    case 5:
        align = CComVariant(xlHAlignFill);
        break;
    }
    (*RHS) = align;

    return hr;
}

HRESULT __stdcall CRange::put_HorizontalAlignment(VARIANT RHS)
{ 
    TRACINGMETHOD; 

    int justify(0);
    CComVariant align(RHS);
    hr = align.ChangeType(VT_I4);
    CHECKERROR;
    switch (align.intVal) 
    {
    case xlHAlignCenter:
        justify = 2;
        break;
    case xlHAlignCenterAcrossSelection:
        justify = 0;
        break;
    case xlHAlignDistributed:
        justify = 0;
        break;
    case xlHAlignFill:
        justify = 5;
        break;
    case xlHAlignGeneral:
        justify = 0;
        break;
    case xlHAlignJustify:
        justify = 4;
        break;
    case xlHAlignLeft:
        justify = 1;
        break;
    case xlHAlignRight:
        justify = 3;
        break;
    }

    hr = m_pRange->put_HoriJustify(justify);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CRange::get_IndentLevel( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_IndentLevel( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::InsertIndent( 
                                       long InsertAmount)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Insert( 
                                 VARIANT Shift,
                                 VARIANT CopyOrigin,
                                 VARIANT *RHS)
{
    TRACINGMETHOD;
    XlInsertShiftDirection action(xlShiftDown);

    CComVariant vCopyOrigin(CopyOrigin);

    CComVariant vShift(Shift);

    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;

    if (vShift.vt == VT_NULL || vShift.vt == VT_EMPTY || vShift.vt == VT_ERROR )
    {
        if ((address.GetRight() - address.GetLeft()) > (address.GetBottom() - address.GetTop()))
        {
            action = xlShiftDown;
        }
    }
    else
    {
        hr = vShift.ChangeType(VT_I4);
        CHECKERROR;
        if (vShift.intVal != xlShiftToRight && vShift.intVal != xlShiftDown)
        {
            action = xlShiftDown;
        }
        else
        {
            action = (XlInsertShiftDirection)vShift.intVal;
        }

    }

    switch(action)
    {
    case xlShiftToRight:
        {
            COOColumnsPtr pColumns(NULL);
            hr = m_pRange->GetColumns(&pColumns);
            CHECKERROR;
            hr = pColumns->InsertByIndex(0, address.GetRight() - address.GetLeft() + 1);
            CHECKERROR;
            break;
        }
    case xlShiftDown:
        {
            COORowsPtr pRows(NULL);
            hr = m_pRange->GetRows(&pRows);
            CHECKERROR;
            hr = pRows->InsertByIndex(0, address.GetBottom() - address.GetTop() + 1);
            CHECKERROR;
            break;
        }
    }

    CComVariant copyOrigin(CopyOrigin);
    HRESULT tryCast = copyOrigin.ChangeType(VT_BOOL);
    if (!SUCCEEDED(tryCast) || CopyOrigin.boolVal != VARIANT_FALSE)
    {
        COODispatchHelperPtr pDispatchHelper(NULL);
        hr = m_ParentApplication->GetOOApplication()->CreateDispatchHelper(&pDispatchHelper);

        IDispatchPtr pFrame2(NULL);
        hr = m_pSheet->GetParentWorkbook()->GetOODocument()->GetDocument()->GetFrame(&pFrame2);
        CHECKERROR;

        hr = pDispatchHelper->ExecuteDispatch(pFrame2, OLESTR(".uno:Paste"));
        CHECKERROR;

    }

    IDispatch* result = this;
    (*RHS) = CComVariant(result);
    (result)->AddRef();

    return hr;
}

HRESULT __stdcall CRange::get_Interior( 
                                       Interior **interior)
{ 
    TRACINGMETHOD; 

    CInteriorObject* pInterior(NULL);
    hr = CInteriorObject::CreateInstance(&pInterior);
    CHECKERROR;
    CInteriorPtr pInteriorPtr(pInterior);
    hr = pInteriorPtr->Initialize(this);
    CHECKERROR;
    (*interior) = pInteriorPtr;
    (*interior)->AddRef();
    return hr; 
}

HRESULT __stdcall CRange::get_Item( 
                                   VARIANT RowIndex,
                                   VARIANT ColumnIndex,
                                   long lcid,
                                   VARIANT *RHS)
{ 
    TRACINGMETHOD; 
    hr = get__Default(RowIndex, ColumnIndex, lcid, RHS);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CRange::put_Item( 
                                   VARIANT RowIndex,
                                   VARIANT ColumnIndex,
                                   long lcid,
                                   VARIANT RHS)
{
    TRACINGMETHOD;
    hr = put__Default(RowIndex, ColumnIndex, lcid, RHS);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CRange::Justify( 
                                  VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Left( 
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;
    (*RHS) = CComVariant(address.GetLeft());

    return hr;
}

HRESULT __stdcall CRange::get_ListHeaderRows( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::ListNames( 
                                    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_LocationInTable( 
    XlLocationInTable *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Locked( 
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    bool locked(false);
    hr = m_pRange->get_IsLocked(&locked);
    CHECKERROR;

    (*RHS) = CComVariant(locked);

    return hr;
}

HRESULT __stdcall CRange::put_Locked( 
                                     VARIANT RHS)
{
    TRACINGMETHOD;

    CComVariant locked(RHS);
    hr = locked.ChangeType(VT_BOOL);
    CHECKERROR;

    hr = m_pRange->put_IsLocked(locked.boolVal != VARIANT_FALSE);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::Merge( 
                                VARIANT Across)
{
    TRACINGMETHOD;

    CComVariant vAcross(Across);

    if ((vAcross.vt == VT_BOOL) && (vAcross.boolVal != VARIANT_FALSE))
    {
        CRangeAddress address(OLESTR("A1:A1"));
        hr = GetRangeAddress(address);
        CHECKERROR;

        for (long row = 0; row <= address.GetBottom() - address.GetTop(); ++row)
        {
            COORangePtr pRange(NULL);
            hr = m_pRange->GetCellRangeByPosition(0, row, address.GetRight() - address.GetLeft(), row, &pRange);
            CHECKERROR;
            hr = pRange->Merge(true);
            CHECKERROR;
        }
    }
    else
    {
        hr = m_pRange->Merge(true);
    }
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::UnMerge( void)
{
    TRACINGMETHOD;

    hr = m_pRange->Merge(false);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::get_MergeArea( 
                                        Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_MergeCells( 
    VARIANT *RHS)
{
    TRACINGMETHOD;

    bool merged(false);
    hr = m_pRange->GetIsMerged(&merged);

    CHECKERROR;

    (*RHS) = CComVariant(merged);

    return hr;
}

HRESULT __stdcall CRange::put_MergeCells( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    CComVariant merged(RHS);
    hr = merged.ChangeType(VT_BOOL);
    CHECKERROR;

    if (merged.boolVal != VARIANT_FALSE)
    {
        hr = Merge(CComVariant(false));
    }
    else
    {
        hr = UnMerge();
    }

    CHECKERROR;
    return hr;
}

HRESULT __stdcall CRange::get_Name( 
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_Name( 
                                   VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::NavigateArrow( 
                                        VARIANT TowardPrecedent,
                                        VARIANT ArrowNumber,
                                        VARIANT LinkNumber,
                                        VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get__NewEnum( 
                                       IUnknown **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Next( 
                                   Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::NoteText( 
                                   VARIANT Text,
                                   VARIANT Start,
                                   VARIANT Length,
                                   BSTR *RHS)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_NumberFormat( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    _Workbook* activeWB(NULL);
    m_ParentApplication->get_ActiveWorkbook(&activeWB);

    CWorkbookPtr pWB = reinterpret_cast<CWorkbook*>(activeWB);
    activeWB->Release();

    // getting all formats 
    COONumberFormatsPtr pNumberFormats(NULL);
    hr = pWB->GetOODocument()->get_NumberFormats(&pNumberFormats);
    CHECKERROR;

    int key(-1);
    hr = m_pRange->get_NumberFormat(&key);
    CHECKERROR;

    CComBSTR format(NULL);
    hr = pNumberFormats->GetByKey(key, &format);
    CHECKERROR;

    (*RHS) = CComVariant(format);

    return hr;
}

HRESULT __stdcall CRange::put_NumberFormat(VARIANT RHS)
{ 
    TRACINGMETHOD; 

    _Workbook* activeWB(NULL);
    m_ParentApplication->get_ActiveWorkbook(&activeWB);

    CWorkbookPtr pWB = reinterpret_cast<CWorkbook*>(activeWB);
    activeWB->Release();

    // getting all formats 
    COONumberFormatsPtr pNumberFormats(NULL);
    hr = pWB->GetOODocument()->get_NumberFormats(&pNumberFormats);
    CHECKERROR;

    // get current locale
    COOCharLocalePtr pCharLocale(NULL);
    hr = m_pRange->get_CharLocale(&pCharLocale);
    CHECKERROR;

    CComVariant formatString(RHS); 
    hr = formatString.ChangeType(VT_BSTR);
    CHECKERROR;
    formatString = CFormatCorrector(formatString.bstrVal).ToString().c_str();
    // check existing same format
    int key(-1);
    hr = pNumberFormats->QueryKey(formatString.bstrVal, pCharLocale, true, &key);
    CHECKERROR;
    if (key == -1)
    {
        hr = pNumberFormats->AddNew(formatString.bstrVal, pCharLocale, &key);
        CHECKERROR;
    }

    hr = m_pRange->put_NumberFormat(key);
    CHECKERROR;
    return hr;

}

HRESULT __stdcall CRange::get_NumberFormatLocal( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    return get_NumberFormat(RHS);
}

HRESULT __stdcall CRange::put_NumberFormatLocal( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    return put_NumberFormat(RHS);
}

HRESULT __stdcall CRange::get_Offset( 
                                     VARIANT RowOffset,
                                     VARIANT ColumnOffset,
                                     Range **RHS)
{ 
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Orientation( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_Orientation( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_OutlineLevel( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_OutlineLevel( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_PageBreak( 
                                        long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_PageBreak( 
                                        long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Parse( 
                                VARIANT ParseLine,
                                VARIANT Destination,
                                VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::_PasteSpecial( 
                                        XlPasteType Paste,
                                        XlPasteSpecialOperation Operation,
                                        VARIANT SkipBlanks,
                                        VARIANT Transpose,
                                        VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_PivotField( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_PivotItem( 
                                        IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_PivotTable( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Precedents( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_PrefixCharacter( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Previous( 
                                       Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::_PrintOut( 
                                    VARIANT From,
                                    VARIANT To,
                                    VARIANT Copies,
                                    VARIANT Preview,
                                    VARIANT ActivePrinter,
                                    VARIANT PrintToFile,
                                    VARIANT Collate,
                                    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::PrintPreview( 
                                       VARIANT EnableChanges,
                                       VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_QueryTable( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Range( 
                                    VARIANT Cell1,
                                    VARIANT Cell2,
                                    Range **RHS)
{ 
    TRACINGMETHOD; 
    CRangeObject* pCells(NULL);
    hr = CRangeObject::CreateInstance(&pCells);
    CHECKERROR;
    CRangePtr pCellsPtr(pCells);

    CComVariant cell1(Cell1);
    CComVariant cell2(Cell2);

    std::auto_ptr<CRangeAddress> address(0);

    if (cell1.vt == VT_BSTR)
    {
        std::wstring wstrAddress = cell1.bstrVal;
        if (cell2.vt == VT_BSTR)
        {
            address.reset(new CRangeAddress(wstrAddress + CRangeAddress::GetSpot() + cell2.bstrVal));
        }
        else
        {
            if (std::wstring::npos == wstrAddress.find(CRangeAddress::GetSpot()))
            {
                address.reset(new CRangeAddress(wstrAddress + CRangeAddress::GetSpot() + wstrAddress));
            }
            else
            {
                address.reset(new CRangeAddress(wstrAddress));
            }
        }
    }
    else
    {
        if (cell1.vt == VT_DISPATCH)
        {
            CRangePtr pRange1(NULL);
            hr = cell1.pdispVal->QueryInterface((Range**)&pRange1);
            CHECKERROR;
            CRangeAddress address1(OLESTR("A1:A1"));
            hr = pRange1->GetRangeAddress(address1);
            CHECKERROR;

            if (cell2.vt == VT_DISPATCH)
            {
                CRangePtr pRange2(NULL);
                hr = cell2.pdispVal->QueryInterface((Range**)&pRange2);
                CHECKERROR;
                CRangeAddress address2(OLESTR("A1:A1"));
                hr = pRange2->GetRangeAddress(address2);
                CHECKERROR;

                address.reset(new CRangeAddress(address1.GetTopLeft() + CPoint(1, 1), address2.GetBottomRight() + CPoint(1, 1)));
            }
            else
            {
                address.reset(new CRangeAddress(address1));
            }
        }
        else
        {
            return E_FAIL;
        }
    }

    hr = pCellsPtr->Initialize(m_ParentApplication, m_pSheet, GetRangeObject(), address->ToString());
    CHECKERROR;

    m_garbageList.AddTail(pCellsPtr.p);

    (*RHS) = pCellsPtr;
    (*RHS)->AddRef();
    return hr; 
}

HRESULT __stdcall CRange::RemoveSubtotal( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Replace( 
                                  VARIANT What,
                                  VARIANT Replacement,
                                  VARIANT LookAt,
                                  VARIANT SearchOrder,
                                  VARIANT MatchCase,
                                  VARIANT MatchByte,
                                  VARIANT SearchFormat,
                                  VARIANT ReplaceFormat,
                                  VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Resize( 
                                     VARIANT RowSize,
                                     VARIANT ColumnSize,
                                     Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Row( 
                                  long *RHS)
{
    TRACINGMETHOD;
    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;

    (*RHS) = address.GetTop() + 1;

    return hr;
}

HRESULT __stdcall CRange::RowDifferences( 
    VARIANT Comparison,
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_RowHeight( 
                                        VARIANT *RHS)
{
    TRACINGMETHOD;
    COORowsPtr pRows(NULL);
    hr = m_pRange->GetRows(&pRows);
    CHECKERROR;

    int height(0);
    hr = pRows->get_Height(&height);
    CHECKERROR;

    double actual = height;
    actual /= 1000;
    actual *= 28;


    (*RHS) = CComVariant((int)actual);

    return hr;
}

HRESULT __stdcall CRange::put_RowHeight( 
                                        VARIANT RHS)
{ 
    TRACINGMETHOD; 

    COORowsPtr pRows(NULL);
    hr = m_pRange->GetRows(&pRows);
    CHECKERROR;

    CComVariant rowHeight(RHS);
    hr = rowHeight.ChangeType(VT_I4);
    CHECKERROR;

    double actual = rowHeight.intVal;
    actual /= 28.0;
    actual *= 1000;

    hr = pRows->put_Height((int)actual);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::get_Rows( 
                                   Range **rows)
{ 
    TRACINGMETHOD; 

    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;
    CRangeObject* pRange(NULL);
    hr = CRangeObject::CreateInstance(&pRange);
    CHECKERROR;
    CRangePtr pRangePtr(pRange);

    COORangePtr sheetPtr(NULL);
    hr = m_pRange->get_Spreadsheet(&sheetPtr);
    CHECKERROR;

    hr = pRangePtr->Initialize(m_ParentApplication, m_pSheet, sheetPtr, CPoint(CAddressItem::MinCol() + 1, address.GetTop() + 1), CPoint(CAddressItem::MaxCol() + 1, address.GetBottom() + 1));
    CHECKERROR;
    (*rows) = pRangePtr;
    (*rows)->AddRef();
    return hr;
}

HRESULT __stdcall CRange::Run( 
                              VARIANT Arg1,
                              VARIANT Arg2,
                              VARIANT Arg3,
                              VARIANT Arg4,
                              VARIANT Arg5,
                              VARIANT Arg6,
                              VARIANT Arg7,
                              VARIANT Arg8,
                              VARIANT Arg9,
                              VARIANT Arg10,
                              VARIANT Arg11,
                              VARIANT Arg12,
                              VARIANT Arg13,
                              VARIANT Arg14,
                              VARIANT Arg15,
                              VARIANT Arg16,
                              VARIANT Arg17,
                              VARIANT Arg18,
                              VARIANT Arg19,
                              VARIANT Arg20,
                              VARIANT Arg21,
                              VARIANT Arg22,
                              VARIANT Arg23,
                              VARIANT Arg24,
                              VARIANT Arg25,
                              VARIANT Arg26,
                              VARIANT Arg27,
                              VARIANT Arg28,
                              VARIANT Arg29,
                              VARIANT Arg30,
                              VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Select( 
                                 VARIANT *pvarResult)
{ 
    TRACINGMETHOD; 
    hr = m_pSheet->GetParentWorkbook()->GetOODocument()->Select(m_pRange);
    CHECKERROR;

    *pvarResult = CComVariant(true);
    return hr;
}

HRESULT __stdcall CRange::Show( 
                               VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::ShowDependents( 
    VARIANT Remove,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_ShowDetail( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_ShowDetail( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::ShowErrors( 
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::ShowPrecedents( 
    VARIANT Remove,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_ShrinkToFit( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    bool bShrinkToFit(false);
    hr = m_pRange->get_ShrinkToFit(&bShrinkToFit);
    CHECKERROR;

    (*RHS) = CComVariant(bShrinkToFit);

    return hr;
}

HRESULT __stdcall CRange::put_ShrinkToFit( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    CComVariant vShrinkToFit(RHS);
    hr = vShrinkToFit.ChangeType(VT_BOOL);
    CHECKERROR;

    hr = m_pRange->put_ShrinkToFit(vShrinkToFit.boolVal != VARIANT_FALSE);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::Sort( 
                               VARIANT Key1,
                               XlSortOrder Order1,
                               VARIANT Key2,
                               VARIANT Type,
                               XlSortOrder Order2,
                               VARIANT Key3,
                               XlSortOrder Order3,
                               XlYesNoGuess Header,
                               VARIANT OrderCustom,
                               VARIANT MatchCase,
                               XlSortOrientation Orientation,
                               XlSortMethod SortMethod,
                               XlSortDataOption DataOption1,
                               XlSortDataOption DataOption2,
                               XlSortDataOption DataOption3,
                               VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::SortSpecial( 
                                      XlSortMethod SortMethod,
                                      VARIANT Key1,
                                      XlSortOrder Order1,
                                      VARIANT Type,
                                      VARIANT Key2,
                                      XlSortOrder Order2,
                                      VARIANT Key3,
                                      XlSortOrder Order3,
                                      XlYesNoGuess Header,
                                      VARIANT OrderCustom,
                                      VARIANT MatchCase,
                                      XlSortOrientation Orientation,
                                      XlSortDataOption DataOption1,
                                      XlSortDataOption DataOption2,
                                      XlSortDataOption DataOption3,
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_SoundNote( 
                                        IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::SpecialCells( 
                                       XlCellType Type,
                                       VARIANT Value,
                                       Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Style( 
                                    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_Style( 
                                    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::SubscribeTo( 
                                      BSTR Edition,
                                      XlSubscribeToFormat Format,
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Subtotal( 
                                   long GroupBy,
                                   XlConsolidationFunction Function,
                                   VARIANT TotalList,
                                   VARIANT Replace,
                                   VARIANT PageBreaks,
                                   XlSummaryRow SummaryBelowData,
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Summary( 
                                      VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Table( 
                                VARIANT RowInput,
                                VARIANT ColumnInput,
                                VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Text( 
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    hr = get_Value(CComVariant(), 0, RHS);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CRange::TextToColumns( 
                                        VARIANT Destination,
                                        XlTextParsingType DataType,
                                        XlTextQualifier TextQualifier,
                                        VARIANT ConsecutiveDelimiter,
                                        VARIANT Tab,
                                        VARIANT Semicolon,
                                        VARIANT Comma,
                                        VARIANT Space,
                                        VARIANT Other,
                                        VARIANT OtherChar,
                                        VARIANT FieldInfo,
                                        VARIANT DecimalSeparator,
                                        VARIANT ThousandsSeparator,
                                        VARIANT TrailingMinusNumbers,
                                        VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Top( 
                                  VARIANT *RHS)
{
    TRACINGMETHOD;
    CRangeAddress address(OLESTR("A1:A1"));
    hr = GetRangeAddress(address);
    CHECKERROR;

    (*RHS) = CComVariant(address.GetTop());

    return hr;
}

HRESULT __stdcall CRange::Ungroup( 
                                  VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_UseStandardHeight( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_UseStandardHeight( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_UseStandardWidth( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_UseStandardWidth( 
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Validation( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Value( 
                                    VARIANT RangeValueDataType,
                                    long lcid,
                                    VARIANT *RHS)
{ 
    TRACINGMETHOD; 

    COOCellPtr pCell(NULL);
    hr = m_pRange->GetCellByPosition(0, 0, &pCell);
    CHECKERROR;

    CComVariant vFormula;
    hr = pCell->GetFormula(&vFormula);
    CHECKERROR;

    (*RHS) = vFormula;

    return hr;
}

HRESULT __stdcall CRange::put_Value( 
                                    VARIANT RangeValueDataType,
                                    long lcid,
                                    VARIANT RHS)
{ 
    TRACINGMETHOD; 

    CComVariant vValue(RHS);

    WRITE_LOG_R_W((L"   > params:"));
    WRITE_LOG_R_W((L"   >> RangeValueDataType = (vt = %d , intVal = %d)", vValue.vt, vValue.intVal));
    WRITE_LOG_R_W((L"   >> lcid               = %d", lcid));
    WRITE_LOG_R_W((L"   >> RHS                = (vt = %d , intVal = %d)", RHS.vt, RHS.intVal));
    WRITE_LOG_R_W((L"   > params"));

    if ((vValue.vt & VT_ARRAY) == VT_ARRAY) 
    {
        WRITE_LOG_R_W((L"   > array type:"));
        if ((vValue.vt ^ VT_ARRAY) == VT_BSTR)
        {
            WRITE_LOG_R_W((L"   >> array subtype is VT_BSTR"));
            hr = SetArrayValue<BSTR>(vValue); 
        }
        if ((vValue.vt ^ VT_ARRAY) == VT_VARIANT)
        {
            WRITE_LOG_R_W((L"   >> array subtype is VT_VARIANT"));
            hr = SetArrayValue<VARIANT>(vValue); 
        }
        WRITE_LOG_R_W((L"   > array type"));
        return hr;
    }
    else if (vValue.vt == VT_NULL)
    {
        WRITE_LOG_R_W((L"   > VT_NULL type: clearing"));
        CRangeAddress address(OLESTR("A1:A1"));
        hr = GetRangeAddress(address);
        CHECKERROR;
        WRITE_LOG_R_W((L"   >> actual address: %s", address.ToString().c_str()));
        for (long i = 0; i < address.GetRight() - address.GetLeft() + 1; ++i)
        {
            for (long j = 0; j < address.GetBottom() - address.GetTop() + 1; ++j)
            {
                COOCellPtr pCell(NULL);
                hr = m_pRange->GetCellByPosition(i, j, &pCell);
                CHECKERROR;
                hr = pCell->ClearContents(1);
                CHECKERROR;
            }
        }
        WRITE_LOG_R_W((L"   > VT_NULL type"));
    }
    else
    {
        WRITE_LOG_R_W((L"   > some other type: seting"));
        CRangeAddress address(OLESTR("A1:A1"));
        hr = GetRangeAddress(address);
        CHECKERROR;
        WRITE_LOG_R_W((L"   >> actual address: %s", address.ToString().c_str()));
        for (long i = 0; i < address.GetRight() - address.GetLeft() + 1; ++i)
        {
            for (long j = 0; j < address.GetBottom() - address.GetTop() + 1; ++j)
            {
                COOCellPtr pCell(NULL);
                hr = m_pRange->GetCellByPosition(i, j, &pCell);
                CHECKERROR;
                VARTYPE type = vValue.vt;
                if ((type & VT_BYREF) == VT_BYREF)
                {
                    type ^= VT_BYREF;
                }
                hr = vValue.ChangeType(type);
                CHECKERROR;
                hr = pCell->put_FormulaLocal(vValue);
                CHECKERROR;
            }
        }
        WRITE_LOG_R_W((L"   > some other type"));
    }
    return hr;
}

HRESULT __stdcall CRange::get_Value2( 
                                     long lcid,
                                     VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_Value2( 
                                     long lcid,
                                     VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_VerticalAlignment( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    int justify(0);
    CComVariant align(0);
    hr = m_pRange->get_VertJustify(&justify);
    switch (justify) 
    {
    case 0:
        align = CComVariant(xlVAlignJustify);
        break;
    case 1:
        align = CComVariant(xlVAlignTop);
        break;
    case 2:
        align = CComVariant(xlVAlignCenter);
        break;
    case 3:
        align = CComVariant(xlVAlignBottom);
        break;
    }

    (*RHS) = align;
    return hr;
}

HRESULT __stdcall CRange::put_VerticalAlignment(VARIANT RHS)
{ 
    TRACINGMETHOD; 

    int justify(-1);
    CComVariant align(RHS);
    hr = align.ChangeType(VT_I4);
    CHECKERROR;

    switch (align.intVal) {
    case xlVAlignBottom:
        justify = 3;
        break;
    case xlVAlignCenter:
        justify = 2;
        break;
    case xlVAlignDistributed:
        justify = 0;
        break;
    case xlVAlignJustify:
        justify = 0;
        break;
    case xlVAlignTop:
        justify = 1;
        break;
    }

    hr = m_pRange->put_VertJustify(justify);
    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::get_Width( 
                                    VARIANT *RHS)
{
    TRACINGMETHOD;
    short width(0);
    hr = m_pRange->get_Width(&width);
    CHECKERROR;

    (*RHS) = CComVariant((width + 1) / 100);

    return hr;
}

HRESULT __stdcall CRange::get_Worksheet( 
                                        _Worksheet **RHS)
{
    TRACINGMETHOD;
    (*RHS) = m_pSheet;
    (*RHS)->AddRef();
    return hr;
}

HRESULT __stdcall CRange::get_WrapText( 
                                       VARIANT *RHS)
{
    TRACINGMETHOD;
    bool wrapState(false);
    hr = m_pRange->get_IsTextWrapped(&wrapState);
    CHECKERROR;
    (*RHS) = CComVariant(wrapState);
    return hr;
}

HRESULT __stdcall CRange::put_WrapText( 
                                       VARIANT RHS)
{
    TRACINGMETHOD;
    CComVariant wrapState(RHS);
    hr = wrapState.ChangeType(VT_BOOL);
    CHECKERROR;
    hr = m_pRange->put_IsTextWrapped(wrapState.boolVal != VARIANT_FALSE);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CRange::AddComment( 
                                     VARIANT Text,
                                     IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Comment( 
                                      IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::ClearComments( void)
{
    TRACINGMETHOD;

    hr = m_pRange->ClearContents(COORange::ANNOTATION);

    CHECKERROR;

    return hr;
}

HRESULT __stdcall CRange::get_Phonetic( 
                                       IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_FormatConditions( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_ReadingOrder( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_ReadingOrder( 
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Hyperlinks( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Phonetics( 
                                        IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::SetPhonetic( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_ID( 
                                 BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::put_ID( 
                                 BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::PrintOut( 
                                   VARIANT From,
                                   VARIANT To,
                                   VARIANT Copies,
                                   VARIANT Preview,
                                   VARIANT ActivePrinter,
                                   VARIANT PrintToFile,
                                   VARIANT Collate,
                                   VARIANT PrToFileName,
                                   VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_PivotCell( 
                                        IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Dirty( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_Errors( 
                                     IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_SmartTags( 
                                        IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::Speak( 
                                VARIANT SpeakDirection,
                                VARIANT SpeakFormulas)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::PasteSpecial( 
                                       XlPasteType Paste,
                                       XlPasteSpecialOperation Operation,
                                       VARIANT SkipBlanks,
                                       VARIANT Transpose,
                                       VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_AllowEdit( 
                                        VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_ListObject( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CRange::get_XPath( 
                                    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}


