
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "Workbooks.h"

using namespace AutomateIT;
using namespace AutomateIT::Utils;
using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::OpenOfficeWrapper;

CWorkbooks::CWorkbooks() : m_ParentApplication(NULL)
{
    TRACINGMETHOD; 


}

CWorkbooks::~CWorkbooks()
{
    TRACINGMETHOD; 
}

HRESULT STDMETHODCALLTYPE CWorkbooks::GetActiveWorkbook(CWorkbook** workbook)
{
    // Active Workbook is last added one.
    TRACINGMETHOD; 
    if (!m_WorkbookList.IsEmpty())
    {
        *workbook = m_WorkbookList.GetTail();
        (*workbook)->AddRef();
        return hr;
    }
    return E_FAIL;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::Initialize(CApplication* pdApp)
{
    TRACINGMETHOD; 
    m_ParentApplication = pdApp;
    hr = Reset();
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::Reset()
{
    TRACINGMETHOD; 
    m_WorkbookList.RemoveAll();
    COOComponentsPtr pComponents(NULL);
    hr = m_ParentApplication->GetOOApplication()->get_Components(&pComponents);
    CHECKERROR;

    COOEnumerationPtr pEnumeration(NULL);
    hr = pComponents->CreateEnumeration(&pEnumeration);
    CHECKERROR;

    bool hasElems(false);
    hr = pEnumeration->HasMoreElements(&hasElems);
    CHECKERROR;
    while (hasElems)
    {
        IDispatchPtr pElement(NULL);
        hr = pEnumeration->NextElement(&pElement);
        CHECKERROR;

        CWorkbookObject* pWorkbookObject(NULL);
        HRESULT hr = CWorkbookObject::CreateInstance(&pWorkbookObject);
        CHECKERROR;

        CWorkbookPtr pWorkbook(pWorkbookObject);
        hr = pWorkbook->Initialize(m_ParentApplication, pElement);
        if (SUCCEEDED(hr))
        {
            m_WorkbookList.AddTail(pWorkbook);

            hr = pEnumeration->HasMoreElements(&hasElems);
            CHECKERROR;
        }
    }

    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::get_Application( 
    _Application **RHS)
{
    TRACINGMETHOD;
    (*RHS) = m_ParentApplication;
    (*RHS)->AddRef();
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::get_Creator( 
    XlCreator *RHS)
{
    TRACINGMETHOD;
    (*RHS) = xlCreatorCode;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::get_Parent( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    (*RHS) = m_ParentApplication;
    (*RHS)->AddRef();
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::Add( 
    VARIANT Template,
    long lcid,
    _Workbook **ppWorkbook)
{ 
    TRACINGMETHOD; 

    CWorkbookObject* pWorkbook(NULL);
    hr = CWorkbookObject::CreateInstance(&pWorkbook);
    CHECKERROR;
    CWorkbookPtr pWorkbookPtr(pWorkbook);
    hr = pWorkbookPtr->Initialize(m_ParentApplication);
    CHECKERROR;

    m_WorkbookList.AddTail(pWorkbookPtr);
    *ppWorkbook = pWorkbookPtr;
    (*ppWorkbook)->AddRef();

    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::Close( 
    long lcid)
{
    TRACINGMETHOD;

    CAtlList<CWorkbookPtr> tempWb;
    tempWb.AddHeadList(&m_WorkbookList);

    POSITION pos = tempWb.GetHeadPosition();
    while( pos != NULL )
    {
        CWorkbookPtr element = tempWb.GetNext( pos );
        hr = element->Close(CComVariant(false), CComVariant(), CComVariant(), lcid);
        CHECKERROR;
    }
    tempWb.RemoveAll();
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::get_Count( 
    long *RHS)
{
    TRACINGMETHOD;
    *RHS = static_cast<long>(m_WorkbookList.GetCount());
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::get_Item( 
    VARIANT Index,
    _Workbook **RHS)
{
    TRACINGMETHOD;
    CComVariant vIndex(Index);
    hr = vIndex.ChangeType(VT_I4);
    CHECKERROR;

    POSITION pos = m_WorkbookList.FindIndex(static_cast<size_t>(vIndex.intVal - 1));
    CWorkbookPtr pWorkbook = m_WorkbookList.GetAt(pos);
    (*RHS) = pWorkbook;
    (*RHS)->AddRef();
    
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::get__NewEnum( 
    IUnknown **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::_Open( 
    BSTR Filename,
    VARIANT UpdateLinks,
    VARIANT ReadOnly,
    VARIANT Format,
    VARIANT Password,
    VARIANT WriteResPassword,
    VARIANT IgnoreReadOnlyRecommended,
    VARIANT Origin,
    VARIANT Delimiter,
    VARIANT Editable,
    VARIANT Notify,
    VARIANT Converter,
    VARIANT AddToMru,
    long lcid,
    _Workbook **ppWorkbook)
{ 
    TRACINGMETHOD; 

    std::wstring sName = Filename;
    std::replace(sName.begin(), sName.end(), L'\\', L'/');
    std::replace(sName.begin(), sName.end(), L':', L'|');
    sName = L"file:///" + sName;

    CWorkbookObject* pWorkbookObject(NULL);
    hr = CWorkbookObject::CreateInstance(&pWorkbookObject);
    CHECKERROR;

    CWorkbookPtr pWorkbookPtr(pWorkbookObject);
    hr = pWorkbookPtr->Initialize(m_ParentApplication, sName.c_str());
    CHECKERROR;

    m_WorkbookList.AddTail(pWorkbookPtr);
    (*ppWorkbook) = pWorkbookPtr;
    (*ppWorkbook)->AddRef();

    return hr;

}

HRESULT STDMETHODCALLTYPE CWorkbooks::__OpenText( 
    BSTR Filename,
    VARIANT Origin,
    VARIANT StartRow,
    VARIANT DataType,
    XlTextQualifier TextQualifier,
    VARIANT ConsecutiveDelimiter,
    VARIANT Tab,
    VARIANT Semicolon,
    VARIANT Comma,
    VARIANT Space,
    VARIANT Other,
    VARIANT OtherChar,
    VARIANT FieldInfo,
    VARIANT TextVisualLayout,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::get__Default( 
    VARIANT Index,
    _Workbook **RHS)
{
    TRACINGMETHOD;
    hr = get_Item(Index, RHS);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::_OpenText( 
    BSTR Filename,
    VARIANT Origin,
    VARIANT StartRow,
    VARIANT DataType,
    XlTextQualifier TextQualifier,
    VARIANT ConsecutiveDelimiter,
    VARIANT Tab,
    VARIANT Semicolon,
    VARIANT Comma,
    VARIANT Space,
    VARIANT Other,
    VARIANT OtherChar,
    VARIANT FieldInfo,
    VARIANT TextVisualLayout,
    VARIANT DecimalSeparator,
    VARIANT ThousandsSeparator,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::Open( 
    BSTR Filename,
    VARIANT UpdateLinks,
    VARIANT ReadOnly,
    VARIANT Format,
    VARIANT Password,
    VARIANT WriteResPassword,
    VARIANT IgnoreReadOnlyRecommended,
    VARIANT Origin,
    VARIANT Delimiter,
    VARIANT Editable,
    VARIANT Notify,
    VARIANT Converter,
    VARIANT AddToMru,
    VARIANT Local,
    VARIANT CorruptLoad,
    long lcid,
    _Workbook **ppWorkbook)
{ 
    TRACINGMETHOD; 
    return _Open( 
        Filename,
        UpdateLinks,
        ReadOnly,
        Format,
        Password,
        WriteResPassword,
        IgnoreReadOnlyRecommended,
        Origin,
        Delimiter,
        Editable,
        Notify,
        Converter,
        AddToMru,
        lcid,
        ppWorkbook
        );
}

HRESULT STDMETHODCALLTYPE CWorkbooks::OpenText( 
    BSTR Filename,
    VARIANT Origin,
    VARIANT StartRow,
    VARIANT DataType,
    XlTextQualifier TextQualifier,
    VARIANT ConsecutiveDelimiter,
    VARIANT Tab,
    VARIANT Semicolon,
    VARIANT Comma,
    VARIANT Space,
    VARIANT Other,
    VARIANT OtherChar,
    VARIANT FieldInfo,
    VARIANT TextVisualLayout,
    VARIANT DecimalSeparator,
    VARIANT ThousandsSeparator,
    VARIANT TrailingMinusNumbers,
    VARIANT Local,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::OpenDatabase( 
    BSTR Filename,
    VARIANT CommandText,
    VARIANT CommandType,
    VARIANT BackgroundQuery,
    VARIANT ImportDataAs,
    _Workbook **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::CheckOut( 
    BSTR Filename)
{
    TRACINGMETHOD;
    // stub
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::CanCheckOut( 
    BSTR Filename,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    // stub
    TRACINGNOTIMPL;
    *RHS = VARIANT_TRUE;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::_OpenXML( 
    BSTR Filename,
    VARIANT Stylesheets,
    _Workbook **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbooks::OpenXML( 
    BSTR Filename,
    VARIANT Stylesheets,
    VARIANT LoadOption,
    _Workbook **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}