
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _WORKBOOKS_H_
#define _WORKBOOKS_H_

#include "ExcelTypes.h"
#include "Workbook.h"
#include <atlcoll.h>

/*
	CWorkbooks class Represents A collection of all the Workbook objects that are currently
	open in the Microsoft Excel application by using OpenOffice.
	It implements IWorkbooks COM interface.
*/

namespace AutomateIT
{
	namespace ExcelClone
	{
		typedef CAtlList<CWorkbookPtr> CWorkbookList;

		class ATL_NO_VTABLE CWorkbooks :
			public CComObjectRootEx<CComSingleThreadModel>,
			public CDispatchHook< IDispatchImpl< Workbooks, &IID_Workbooks, &LIBID_ExcelCloneLib, 1, 0> >
		{
		public:
			CWorkbooks();
			virtual ~CWorkbooks();

			BEGIN_COM_MAP(CWorkbooks)
				COM_INTERFACE_ENTRY(Workbooks)
				COM_INTERFACE_ENTRY(IDispatch)
			END_COM_MAP()

		public:


            HRESULT STDMETHODCALLTYPE get_Application( 
                 _Application **RHS);

            HRESULT STDMETHODCALLTYPE get_Creator( 
                 XlCreator *RHS);

            HRESULT STDMETHODCALLTYPE get_Parent( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Add( 
                 VARIANT Template,
                 long lcid,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE Close( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Count( 
                 long *RHS);

            HRESULT STDMETHODCALLTYPE get_Item( 
                 VARIANT Index,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE get__NewEnum( 
                 IUnknown **RHS);

            HRESULT STDMETHODCALLTYPE _Open( 
                 BSTR Filename,
                 VARIANT UpdateLinks,
                 VARIANT ReadOnly,
                 VARIANT Format,
                 VARIANT Password,
                 VARIANT WriteResPassword,
                 VARIANT IgnoreReadOnlyRecommended,
                 VARIANT Origin,
                 VARIANT Delimiter,
                 VARIANT Editable,
                 VARIANT Notify,
                 VARIANT Converter,
                 VARIANT AddToMru,
                 long lcid,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE __OpenText( 
                 BSTR Filename,
                 VARIANT Origin,
                 VARIANT StartRow,
                 VARIANT DataType,
                 XlTextQualifier TextQualifier,
                 VARIANT ConsecutiveDelimiter,
                 VARIANT Tab,
                 VARIANT Semicolon,
                 VARIANT Comma,
                 VARIANT Space,
                 VARIANT Other,
                 VARIANT OtherChar,
                 VARIANT FieldInfo,
                 VARIANT TextVisualLayout,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get__Default( 
                 VARIANT Index,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE _OpenText( 
                 BSTR Filename,
                 VARIANT Origin,
                 VARIANT StartRow,
                 VARIANT DataType,
                 XlTextQualifier TextQualifier,
                 VARIANT ConsecutiveDelimiter,
                 VARIANT Tab,
                 VARIANT Semicolon,
                 VARIANT Comma,
                 VARIANT Space,
                 VARIANT Other,
                 VARIANT OtherChar,
                 VARIANT FieldInfo,
                 VARIANT TextVisualLayout,
                 VARIANT DecimalSeparator,
                 VARIANT ThousandsSeparator,
                 long lcid);

            HRESULT STDMETHODCALLTYPE Open( 
                 BSTR Filename,
                 VARIANT UpdateLinks,
                 VARIANT ReadOnly,
                 VARIANT Format,
                 VARIANT Password,
                 VARIANT WriteResPassword,
                 VARIANT IgnoreReadOnlyRecommended,
                 VARIANT Origin,
                 VARIANT Delimiter,
                 VARIANT Editable,
                 VARIANT Notify,
                 VARIANT Converter,
                 VARIANT AddToMru,
                 VARIANT Local,
                 VARIANT CorruptLoad,
                 long lcid,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE OpenText( 
                 BSTR Filename,
                 VARIANT Origin,
                 VARIANT StartRow,
                 VARIANT DataType,
                 XlTextQualifier TextQualifier,
                 VARIANT ConsecutiveDelimiter,
                 VARIANT Tab,
                 VARIANT Semicolon,
                 VARIANT Comma,
                 VARIANT Space,
                 VARIANT Other,
                 VARIANT OtherChar,
                 VARIANT FieldInfo,
                 VARIANT TextVisualLayout,
                 VARIANT DecimalSeparator,
                 VARIANT ThousandsSeparator,
                 VARIANT TrailingMinusNumbers,
                 VARIANT Local,
                 long lcid);

            HRESULT STDMETHODCALLTYPE OpenDatabase( 
                 BSTR Filename,
                 VARIANT CommandText,
                 VARIANT CommandType,
                 VARIANT BackgroundQuery,
                 VARIANT ImportDataAs,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE CheckOut( 
                 BSTR Filename);

            HRESULT STDMETHODCALLTYPE CanCheckOut( 
                 BSTR Filename,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE _OpenXML( 
                 BSTR Filename,
                 VARIANT Stylesheets,
                 _Workbook **RHS);

            HRESULT STDMETHODCALLTYPE OpenXML( 
                 BSTR Filename,
                 VARIANT Stylesheets,
                 VARIANT LoadOption,
                 _Workbook **RHS);

		public:

            STDMETHOD(Initialize)(CApplication* pdApp);
			STDMETHOD(GetActiveWorkbook)(CWorkbook** workbook);
            STDMETHOD(Reset)();

		protected:
			CWorkbookList m_WorkbookList;
			CApplication* m_ParentApplication;
		};

		typedef CComObject<CWorkbooks> CWorkbooksObject;
        typedef CComPtr<CWorkbooks> CWorkbooksPtr;
	}
}

#endif //_WORKBOOKS_H_