
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _WORKSHEET_H_
#define _WORKSHEET_H_

#pragma once

#include "Range.h"
#include "OORange.h"

/*
	CWorksheet class represents Excel worksheet object by using OpenOffice.
	It implements IWorksheet COM interface.
*/

namespace AutomateIT
{
    namespace CalcItems
    {
        class CRangeAddress;
    }

	namespace ExcelClone
	{
        class CWorkbook;

		class ATL_NO_VTABLE CWorksheet :
			public CComObjectRootEx<CComSingleThreadModel>,
			public CDispatchHook<IDispatchImpl<_Worksheet, &IID__Worksheet, &LIBID_ExcelCloneLib, 1, 0> >
		{
		public:
			CWorksheet();
			virtual ~CWorksheet();

			BEGIN_COM_MAP(CWorksheet)
				COM_INTERFACE_ENTRY(_Worksheet)
				COM_INTERFACE_ENTRY(IDispatch)
			END_COM_MAP()

		public:

            HRESULT STDMETHODCALLTYPE get_Application(
                _Application **RHS);

            HRESULT STDMETHODCALLTYPE get_Creator(
                 XlCreator *RHS);

            HRESULT STDMETHODCALLTYPE get_Parent( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Activate( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE Copy( 
                 VARIANT Before,
                 VARIANT After,
                 long lcid);

            HRESULT STDMETHODCALLTYPE Delete( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_CodeName( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE get__CodeName( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put__CodeName( 
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_Index( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE Move( 
                 VARIANT Before,
                 VARIANT After,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Name( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_Name( 
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_Next( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_OnDoubleClick( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnDoubleClick( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnSheetActivate( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnSheetActivate( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnSheetDeactivate( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnSheetDeactivate( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_PageSetup( 
                 PageSetup **RHS);

            HRESULT STDMETHODCALLTYPE get_Previous( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE _PrintOut( 
                 VARIANT From,
                 VARIANT To,
                 VARIANT Copies,
                 VARIANT Preview,
                 VARIANT ActivePrinter,
                 VARIANT PrintToFile,
                 VARIANT Collate,
                 long lcid);

            HRESULT STDMETHODCALLTYPE PrintPreview( 
                 VARIANT EnableChanges,
                 long lcid);

            HRESULT STDMETHODCALLTYPE _Protect( 
                 VARIANT Password,
                 VARIANT DrawingObjects,
                 VARIANT Contents,
                 VARIANT Scenarios,
                 VARIANT UserInterfaceOnly,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_ProtectContents( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_ProtectDrawingObjects( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_ProtectionMode( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE get_ProtectScenarios( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE _SaveAs( 
                 BSTR Filename,
                 VARIANT FileFormat,
                 VARIANT Password,
                 VARIANT WriteResPassword,
                 VARIANT ReadOnlyRecommended,
                 VARIANT CreateBackup,
                 VARIANT AddToMru,
                 VARIANT TextCodepage,
                 VARIANT TextVisualLayout,
                 long lcid);

            HRESULT STDMETHODCALLTYPE Select( 
                 VARIANT Replace,
                 long lcid);

            HRESULT STDMETHODCALLTYPE Unprotect( 
                 VARIANT Password,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Visible( 
                 long lcid,
                 XlSheetVisibility *RHS);

            HRESULT STDMETHODCALLTYPE put_Visible( 
                 long lcid,
                 XlSheetVisibility RHS);

            HRESULT STDMETHODCALLTYPE get_Shapes( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_TransitionExpEval( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_TransitionExpEval( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Arcs( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_AutoFilterMode( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_AutoFilterMode( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE SetBackgroundPicture( 
                 BSTR Filename);

            HRESULT STDMETHODCALLTYPE Buttons( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Calculate( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_EnableCalculation( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableCalculation( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Cells( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE ChartObjects( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE CheckBoxes( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE CheckSpelling( 
                 VARIANT CustomDictionary,
                 VARIANT IgnoreUppercase,
                 VARIANT AlwaysSuggest,
                 VARIANT SpellLang,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_CircularReference( 
                 long lcid,
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE ClearArrows( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Columns( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_ConsolidationFunction( 
                 long lcid,
                 XlConsolidationFunction *RHS);

            HRESULT STDMETHODCALLTYPE get_ConsolidationOptions( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_ConsolidationSources( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayAutomaticPageBreaks( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayAutomaticPageBreaks( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Drawings( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE DrawingObjects( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE DropDowns( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_EnableAutoFilter( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableAutoFilter( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_EnableSelection( 
                 XlEnableSelection *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableSelection( 
                 XlEnableSelection RHS);

            HRESULT STDMETHODCALLTYPE get_EnableOutlining( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnableOutlining( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_EnablePivotTable( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_EnablePivotTable( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE Evaluate( 
                 VARIANT Name,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE _Evaluate( 
                 VARIANT Name,
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE get_FilterMode( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE ResetAllPageBreaks( void);

            HRESULT STDMETHODCALLTYPE GroupBoxes( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE GroupObjects( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Labels( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Lines( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE ListBoxes( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Names( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE OLEObjects( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_OnCalculate( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnCalculate( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnData( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnData( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE get_OnEntry( 
                 long lcid,
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_OnEntry( 
                 long lcid,
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE OptionButtons( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Outline( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Ovals( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Paste( 
                 VARIANT Destination,
                 VARIANT Link,
                 long lcid);

            HRESULT STDMETHODCALLTYPE _PasteSpecial( 
                 VARIANT Format,
                 VARIANT Link,
                 VARIANT DisplayAsIcon,
                 VARIANT IconFileName,
                 VARIANT IconIndex,
                 VARIANT IconLabel,
                 long lcid);

            HRESULT STDMETHODCALLTYPE Pictures( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE PivotTables( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE PivotTableWizard( 
                 VARIANT SourceType,
                 VARIANT SourceData,
                 VARIANT TableDestination,
                 VARIANT TableName,
                 VARIANT RowGrand,
                 VARIANT ColumnGrand,
                 VARIANT SaveData,
                 VARIANT HasAutoFormat,
                 VARIANT AutoPage,
                 VARIANT Reserved,
                 VARIANT BackgroundQuery,
                 VARIANT OptimizeCache,
                 VARIANT PageFieldOrder,
                 VARIANT PageFieldWrapCount,
                 VARIANT ReadData,
                 VARIANT Connection,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Range( 
                 VARIANT Cell1,
                 VARIANT Cell2,
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE Rectangles( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Rows( 
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE Scenarios( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_ScrollArea( 
                 BSTR *RHS);

            HRESULT STDMETHODCALLTYPE put_ScrollArea( 
                 BSTR RHS);

            HRESULT STDMETHODCALLTYPE ScrollBars( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE ShowAllData( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE ShowDataForm( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE Spinners( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_StandardHeight( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE get_StandardWidth( 
                 long lcid,
                 double *RHS);

            HRESULT STDMETHODCALLTYPE put_StandardWidth( 
                 long lcid,
                 double RHS);

            HRESULT STDMETHODCALLTYPE TextBoxes( 
                 VARIANT Index,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_TransitionFormEntry( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_TransitionFormEntry( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Type( 
                 long lcid,
                 XlSheetType *RHS);

            HRESULT STDMETHODCALLTYPE get_UsedRange( 
                 long lcid,
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE get_HPageBreaks( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_VPageBreaks( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_QueryTables( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayPageBreaks( 
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayPageBreaks( 
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Comments( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Hyperlinks( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE ClearCircles( void);

            HRESULT STDMETHODCALLTYPE CircleInvalid( void);

            HRESULT STDMETHODCALLTYPE get__DisplayRightToLeft( 
                 long lcid,
                 long *RHS);

            HRESULT STDMETHODCALLTYPE put__DisplayRightToLeft( 
                 long lcid,
                 long RHS);

            HRESULT STDMETHODCALLTYPE get_AutoFilter( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_DisplayRightToLeft( 
                 long lcid,
                 VARIANT_BOOL *RHS);

            HRESULT STDMETHODCALLTYPE put_DisplayRightToLeft( 
                 long lcid,
                 VARIANT_BOOL RHS);

            HRESULT STDMETHODCALLTYPE get_Scripts( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE PrintOut( 
                 VARIANT From,
                 VARIANT To,
                 VARIANT Copies,
                 VARIANT Preview,
                 VARIANT ActivePrinter,
                 VARIANT PrintToFile,
                 VARIANT Collate,
                 VARIANT PrToFileName,
                 long lcid);

            HRESULT STDMETHODCALLTYPE _CheckSpelling( 
                 VARIANT CustomDictionary,
                 VARIANT IgnoreUppercase,
                 VARIANT AlwaysSuggest,
                 VARIANT SpellLang,
                 VARIANT IgnoreFinalYaa,
                 VARIANT SpellScript,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Tab( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_MailEnvelope( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE SaveAs( 
                 BSTR Filename,
                 VARIANT FileFormat,
                 VARIANT Password,
                 VARIANT WriteResPassword,
                 VARIANT ReadOnlyRecommended,
                 VARIANT CreateBackup,
                 VARIANT AddToMru,
                 VARIANT TextCodepage,
                 VARIANT TextVisualLayout,
                 VARIANT Local);

            HRESULT STDMETHODCALLTYPE get_CustomProperties( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_SmartTags( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Protection( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE PasteSpecial( 
                 VARIANT Format,
                 VARIANT Link,
                 VARIANT DisplayAsIcon,
                 VARIANT IconFileName,
                 VARIANT IconIndex,
                 VARIANT IconLabel,
                 VARIANT NoHTMLFormatting,
                 long lcid);

            HRESULT STDMETHODCALLTYPE Protect( 
                 VARIANT Password,
                 VARIANT DrawingObjects,
                 VARIANT Contents,
                 VARIANT Scenarios,
                 VARIANT UserInterfaceOnly,
                 VARIANT AllowFormattingCells,
                 VARIANT AllowFormattingColumns,
                 VARIANT AllowFormattingRows,
                 VARIANT AllowInsertingColumns,
                 VARIANT AllowInsertingRows,
                 VARIANT AllowInsertingHyperlinks,
                 VARIANT AllowDeletingColumns,
                 VARIANT AllowDeletingRows,
                 VARIANT AllowSorting,
                 VARIANT AllowFiltering,
                 VARIANT AllowUsingPivotTables);

            HRESULT STDMETHODCALLTYPE get_ListObjects( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE XmlDataQuery( 
                 BSTR XPath,
                 VARIANT SelectionNamespaces,
                 VARIANT Map,
                 Range **RHS);

            HRESULT STDMETHODCALLTYPE XmlMapQuery( 
                 BSTR XPath,
                 VARIANT SelectionNamespaces,
                 VARIANT Map,
                 Range **RHS);


		public:
        
            HRESULT Initialize(CApplication* pParentApplication, CWorkbook* pParentWorkbook, OpenOfficeWrapper::COORangePtr pSheet);
            CWorkbook* GetParentWorkbook();


        private:


			CRangePtr m_allCells;
			OpenOfficeWrapper::COORangePtr m_pSheet;
            CWorkbook* m_pParentWorkbook;
			CApplication* m_ParentApplication;

        };

	}
}

#endif  //_WORKSHEET_H_
