
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "OOApplication.h"
#include "OOWriterDocument.h"
#include "PropertyValue.h"


using namespace AutomateIT::Utils;

const BYTE COOWriterDocument::FieldTypeText = 0; 

/*
	COOWriterDocument class implements document functionality ( modifications, creation, fields operation etc.)
	NOTE: Currently it implements only small subset of document functionality.
*/

COOWriterDocument::COOWriterDocument() :
	m_pDoc(NULL),
	m_dwCurrBookmark(0)
{
}

COOWriterDocument::~COOWriterDocument()
{
}

COODocument* COOWriterDocument::Attach	(COODocument *pDoc)
{
	COODocument *pOldDoc = m_pDoc;
	m_pDoc = pDoc;
	return pOldDoc;
}

COODocument* COOWriterDocument::Detach ()
{
	COODocument *pOldDoc = m_pDoc;
	m_pDoc = NULL;
	return pOldDoc;
}

HRESULT COOWriterDocument::FileOpen(CComVariant Name)
{
	
	return S_OK;
}
HRESULT COOWriterDocument::FileNewDefault(BOOL bShowed)
{
	if (m_pDoc == NULL)
	{
		return E_FAIL;
	}
	return m_pDoc->m_pApp->CreateBlankDocument(COODocument::TypeWriterDocument, NULL, bShowed);
}

HRESULT COOWriterDocument::FormFieldOptions( 
    VARIANT Name,
    VARIANT Enable,
    VARIANT TextType,
    VARIANT TextWidth,
    VARIANT TextDefault,
    VARIANT TextFormat,
    VARIANT CheckSize,
    VARIANT CheckWidth,
    VARIANT CheckDefault,
    VARIANT Type,
    VARIANT OwnHelp,
    VARIANT HelpText,
    VARIANT OwnStat,
    VARIANT StatText
    )
{
	return S_OK;
}
HRESULT COOWriterDocument::GetCursor	(IDispatch *pdTextSource, IDispatch **ppdCursor)
{
//Set objCursor= objText.createTextCursor
	HRESULT hres;
	if ((ppdCursor == NULL) || (pdTextSource == NULL))
		return E_FAIL;
	CComVariant vRes;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextSource, L"createTextCursor", 0);
	if (hres == S_OK)
	{
		*ppdCursor = vRes.pdispVal;
		(*ppdCursor)->AddRef();
	}
	else 
		*ppdCursor = NULL;
	return hres;
}

HRESULT COOWriterDocument::GetText				(IDispatch **ppdText)
{
//Set objText = objDocument.getText
	HRESULT hres;
	if ((ppdText == NULL) || (m_pDoc == NULL) || (m_pDoc->m_pdDoc == NULL))
		return E_FAIL;

	CComVariant vRes;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"getText", 0);
	if (hres == S_OK)
	{
		*ppdText = vRes.pdispVal;
		(*ppdText)->AddRef();
	}
	else 
		*ppdText = NULL;
	return hres;
}

HRESULT COOWriterDocument::GetCursor			(IDispatch **ppdCursor)
{
//Set objCursor= objText.createTextCursor
	IDispatch *pdText;
	HRESULT hres;
	hres = GetText(&pdText);
	if (hres != S_OK)
		return hres;
	hres = COOWriterDocument::GetCursor(pdText, ppdCursor);
	pdText->Release();
	return hres;
}



HRESULT COOWriterDocument::InsertInputField(CComVariant vName, CComVariant vHint, CComVariant vValue)
{
// TODO: In this time function correctly worked only with text fields, if be needed i todo this all field types

//	HRESULT		hres = S_OK;
//	VARIANT		vRes;
//	if ((m_pDoc == NULL) || (m_pDoc->m_pdDoc == NULL))
//		return E_FAIL;
//
////Set InputField = objDocument.createInstance("com.sun.star.text.TextField.Input")
//	CComVariant p1 = L"com.sun.star.text.TextField.Input";
//	CComVariant p2;
//	CComVariant p3;
//	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"createInstance", 1, p1);
//	if (hres != S_OK)
//		return E_FAIL;
//	IDispatch *pdInputField = vRes.pdispVal;
//	hres = pdInputField->AddRef();
//
////InputField.Hint = "Some Hint" 
//	hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdInputField, L"Hint", 1, vHint);
//	if (hres != S_OK)
//	{
//		pdInputField->Release();
//		return hres;
//	}
////InputField.Content = "Some Value"
// 	hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdInputField, L"Content", 1, vValue);	
//	if (hres != S_OK)
//	{
//		pdInputField->Release();
//		return hres;
//	}
//	IDispatch *pdText;
//	hres = GetText(&pdText);
//	if (hres != S_OK)
//	{
//		pdInputField->Release();
//		return hres;
//	}	
//	IDispatch *pdCursor;
//	hres = COOWriterDocument::GetCursor(pdText, &pdCursor);
//	if (hres != S_OK)
//	{
//		pdText->Release();
//		pdInputField->Release();
//		return hres;
//	}	
////	Text.insertTextContent Cursor, UserField, false
//	p1.vt = VT_DISPATCH;
//	p1.pdispVal = pdCursor;
//
//	p2.vt = VT_DISPATCH;
//	p2.pdispVal = pdInputField;
//
//	p3.vt = VT_BOOL;	
//	p3.boolVal = VARIANT_TRUE;
//	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdText, L"insertTextContent", 3, p3, p2, p1);
//
//	pdText->Release();
//	p1.Clear();
//	p2.Clear();
//	p3.Clear();


	HRESULT		hres = S_OK;
	VARIANT		vRes;
	if ((m_pDoc == NULL) || (m_pDoc->m_pdDoc == NULL))
		return E_FAIL;

//objDocument.Text.insertTextContent Cursor, InputField, false

//Set bookmark = objDocument.createInstance("com.sun.star.text.Bookmark")
	CComVariant p1 = L"com.sun.star.text.Bookmark";
	CComVariant p2;
	CComVariant p3;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"createInstance", 1, p1);
	if (hres != S_OK)
		return E_FAIL;
	IDispatch *pdBookmark = vRes.pdispVal;

//bookmark.setName ("huy_huy")
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"setName", 1, vName);
	if (hres != S_OK)
	{
		pdBookmark->Release();
		return hres;
	}

	IDispatch *pdText;
	hres = GetText(&pdText);
	if (hres != S_OK)
	{
		pdBookmark->Release();
		return hres;
	}	
	IDispatch *pdCursor;
	hres = COOWriterDocument::GetViewCursor(&pdCursor);
	if (hres != S_OK)
	{
		pdText->Release();
		pdBookmark->Release();
		return hres;
	}	

//
	p1.vt = VT_DISPATCH;
	p1.pdispVal = pdCursor;

	p3.vt = VT_BOOL;	
	p3.boolVal = VARIANT_TRUE;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdText, L"insertString", 3, p3, vValue, p1);
	if (hres != S_OK)
	{
		pdText->Release();
		pdCursor->Release();
		pdBookmark->Release();
		return hres;
	}
	
//	Text.insertTextContent Cursor, UserField, false
	p1.vt = VT_DISPATCH;
	p1.pdispVal = pdCursor;

	p2.vt = VT_DISPATCH;
	p2.pdispVal = pdBookmark;

	p3.vt = VT_BOOL;	
	p3.boolVal = VARIANT_TRUE;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdText, L"insertTextContent", 3, p3, p2, p1);
	pdBookmark->Release();
	if (hres != S_OK)
	{
		pdText->Release();
		pdCursor->Release();
		pdBookmark->Release();
		return hres;
	}
//objDocument.Text.InsertString Cursor, "1111111111111111111111111", true
	p1.vt = VT_DISPATCH;
	p1.pdispVal = pdCursor;

	VariantInit(&p2);
	p2 = L"";

	p3.vt = VT_BOOL;	
	p3.boolVal = VARIANT_FALSE;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdText, L"insertString", 3, p3, p2, p1);
	pdText->Release();
	pdCursor->Release();

	VariantInit(&p1);
	VariantInit(&p2);
	VariantInit(&p3);

	//hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"GetAnchor", 0);
	//pdBookmark->Release();
	//if (hres != S_OK)
	//{
	//	return hres;
	//}
	//pdTextRange = vRes.pdispVal;
	//VariantInit(&vRes);
	//hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextRange, L"setString", 1, vValue);
//	pdTextRange->Release();
//objDocument.Text.insertTextContent Cursor, bookmark, true


	//pdCursor->Release();
	//pdText->Release();
	//p1.Clear();
	//p2.Clear();
	//p3.Clear();
	return hres;

	
}

HRESULT COOWriterDocument::InsertTextField(CComVariant vName, CComVariant vValue)
{
// TODO: In this time function correctly worked only with text fields, if be needed i todo this all field types

	HRESULT		hres = S_OK;
	VARIANT		vRes;
	if ((m_pDoc == NULL) || (m_pDoc->m_pdDoc == NULL))
		return E_FAIL;

//Set UserField = objDocument.createInstance("com.sun.star.text.TextField.User")  'com.sun.star.text.TextField.User
	CComVariant p1 = L"com.sun.star.text.TextField.User";
	CComVariant p2;
	CComVariant p3;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"createInstance", 1, p1);
	if (hres != S_OK)
		return E_FAIL;
	IDispatch *pdUserField = vRes.pdispVal;
	hres = pdUserField->AddRef();

//Set UserFieldMaster = objDocument.createInstance("com.sun.star.text.FieldMaster.User")
	p1 = L"com.sun.star.text.FieldMaster.User";
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"createInstance", 1, p1);
	if (hres != S_OK)
	{
		pdUserField->Release();
		return hres;
	}
	IDispatch *pdUserFieldMaster = vRes.pdispVal;
	hres = pdUserFieldMaster->AddRef();

//Set PropertyBeanValue = UserFieldMaster.Bridge_GetStruct("com.sun.star.beans.PropertyValue")
	CPropertyValue *pProperty = new CPropertyValue(pdUserFieldMaster);
	if (pProperty == NULL || pProperty->IsValid() == FALSE)
	{
		pdUserField->Release();
		pdUserFieldMaster->Release();
		return E_FAIL;
	}


//UserFieldMaster.Name = "Some Name" 
	hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdUserFieldMaster, L"Name", 1, vName);
	if (hres != S_OK)
	{
		pdUserField->Release();
		pdUserFieldMaster->Release();
		return hres;
	}
//UserFieldMaster.Value = "Some Value"
 	hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdUserFieldMaster, L"Content", 1, vValue);	
	if (hres != S_OK)
	{
		pdUserField->Release();
		pdUserFieldMaster->Release();
		return hres;
	}

//UserField.attachTextFieldMaster(UserFieldMaster) 
	VariantInit(&p1);
	p1.vt = VT_DISPATCH;
	p1.pdispVal = pdUserFieldMaster;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdUserField, L"attachTextFieldMaster", 1, p1);
	if (hres != S_OK)
	{
		pdUserField->Release();
		pdUserFieldMaster->Release();
		return hres;
	}	
	IDispatch *pdText;
	hres = GetText(&pdText);
	if (hres != S_OK)
	{
		pdUserField->Release();
		pdUserFieldMaster->Release();
		return hres;
	}	
	IDispatch *pdCursor;
	hres = COOWriterDocument::GetCursor(pdText, &pdCursor);
	if (hres != S_OK)
	{
		pdUserField->Release();
		pdUserFieldMaster->Release();
		return hres;
	}	
//	Text.insertTextContent Cursor, UserField, false
	p1.vt = VT_DISPATCH;
	p1.pdispVal = pdCursor;

	p2.vt = VT_DISPATCH;
	p2.pdispVal = pdUserField;

	p3.vt = VT_BOOL;	
	p3.boolVal = VARIANT_TRUE;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdText, L"insertTextContent", 3, p3, p2, p1);

	//pdUserField->Release();
	pdUserFieldMaster->Release();
	pdText->Release();
	//pdCursor->Release();
	p1.Clear();
	p2.Clear();
	p3.Clear();
	return hres;
}

HRESULT COOWriterDocument::SetBookmarkText(CComVariant vBookmarkName, CComVariant vText)
{
	if (m_pDoc == NULL)
		return E_FAIL;
	CComVariant vRes;
	HRESULT hres;
	IDispatch *pdBookmarks;
//Set Bookmarks = objDocument.getBookmarks()
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"getBookmarks", 0);
	if (hres != S_OK)
	{
		return hres;
	}
	pdBookmarks = vRes.pdispVal;
//Set Bookmark = Bookmarks.GetByName("SOTA2")
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmarks, L"getByName", 1, vBookmarkName);
	pdBookmarks->Release();
	pdBookmarks = NULL;
	if (hres != S_OK)
		return hres;
	IDispatch *pdBookmark;
	pdBookmark = vRes.pdispVal;
//Set TextRange = Bookmark.GetAnchor()
	IDispatch *pdTextRange;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"GetAnchor", 0);
	pdBookmark->Release();
	if (hres != S_OK)
		return hres;
	pdTextRange = vRes.pdispVal;
//TextRange.setString("1111111111111111111111111111111111111111111111111111111111111")
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextRange, L"setString", 1, vText);
	pdTextRange->Release();
	return hres;
}

HRESULT COOWriterDocument::GetBookmarkText(CComVariant vBookmarkName, CComVariant *vText)
{
	if (m_pDoc == NULL)
		return E_FAIL;
	CComVariant vRes;
	HRESULT hres;
	IDispatch *pdBookmarks;
//Set Bookmarks = objDocument.getBookmarks()
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"getBookmarks", 0);
	if (hres != S_OK)
	{
		return hres;
	}
	pdBookmarks = vRes.pdispVal;
//Set Bookmark = Bookmarks.GetByName("SOTA2")
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmarks, L"getByName", 1, vBookmarkName);
	pdBookmarks->Release();
	pdBookmarks = NULL;
	if (hres != S_OK)
		return hres;
	IDispatch *pdBookmark;
	pdBookmark = vRes.pdispVal;
//Set TextRange = Bookmark.GetAnchor()
	IDispatch *pdTextRange;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"GetAnchor", 0);
	pdBookmark->Release();
	if (hres != S_OK)
		return hres;
	pdTextRange = vRes.pdispVal;
//TextRange.setString("11111111111111")
	hres = AutoWrapper::Execute(DISPATCH_METHOD, vText, pdTextRange, L"getString", 0);
	vRes.Clear();
	return hres;
}

HRESULT COOWriterDocument::GetBookmarksCount(DWORD &dwCount)
{
	if (m_pDoc == NULL)
	{
		return E_FAIL;
	}
	CComVariant vRes;
	HRESULT hres;
	IDispatch *pdBookmarks;
//Set Bookmarks = objDocument.getBookmarks()
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"getBookmarks", 0);
	if (hres != S_OK)
	{
		return hres;
	}
	pdBookmarks = vRes.pdispVal;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmarks, L"getCount", 0);
	pdBookmarks->Release();
	if (hres != S_OK)
	{
		return hres;
	}
	dwCount = vRes.intVal;
	return S_OK;

}

DWORD	COOWriterDocument::GetCurrentBookmarkIndex		 () const
{
	return m_dwCurrBookmark;
}

HRESULT COOWriterDocument::SelectBookmarkByIndex		 (DWORD dwBookmarkIndex)
{
	DWORD dwBookmarksCount = 0;
	HRESULT hres = GetBookmarksCount(dwBookmarksCount);
	if (hres != S_OK)
	{
		return hres;
	}
	if (m_dwCurrBookmark < dwBookmarksCount)
	{
		m_dwCurrBookmark = dwBookmarkIndex;
		return S_OK;
	}
	else 
	{	
		m_dwCurrBookmark = 0;
		return S_OK;
	}
}

HRESULT COOWriterDocument::SetCurrentBookmarkProperties (CComVariant vBookmarkName, CComVariant vText)
{
	if (m_pDoc == NULL)
		return E_FAIL;
	CComVariant vRes;
	HRESULT hres;
	IDispatch *pdBookmarks;
//Set Bookmarks = objDocument.getBookmarks()
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"getBookmarks", 0);
	if (hres != S_OK)
	{
		return hres;
	}
	pdBookmarks = vRes.pdispVal;
//Set Bookmark = Bookmarks.GetByName("SOTA2")
	CComVariant vBookmarkIndex = m_dwCurrBookmark - 1;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmarks, L"getByIndex", 1, vBookmarkIndex);
	pdBookmarks->Release();
	pdBookmarks = NULL;
	if (hres != S_OK)
	{
		return hres;
	}
	IDispatch *pdBookmark;
	pdBookmark = vRes.pdispVal;
	if (vBookmarkName.vt == VT_BSTR)
	{
		hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"setName", 1, vBookmarkName);
	}

	if (/*vText.vt == VT_BSTR*/ 0)
	{
//Set TextRange = Bookmark.GetAnchor()
		IDispatch *pdTextRange;
		hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"GetAnchor", 0);
		if (hres != S_OK)
		{
			pdBookmark->Release();
			return hres;
		}
		pdTextRange = vRes.pdispVal;
//TextRange.setString("11111111111111")
		hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextRange, L"setString", 1, vText);
		pdTextRange->Release();
	}
	pdBookmark->Release();
	return S_OK;	
}

HRESULT COOWriterDocument::GetCurrentBookmarkProperties (CComVariant *vBookmarkName, CComVariant *vText)
{
	if (m_pDoc == NULL)
		return E_FAIL;
	CComVariant vRes;
	HRESULT hres;
	IDispatch *pdBookmarks;
//Set Bookmarks = objDocument.getBookmarks()
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pDoc->m_pdDoc, L"getBookmarks", 0);
	if (hres != S_OK)
	{
		return hres;
	}
	pdBookmarks = vRes.pdispVal;
//Set Bookmark = Bookmarks.GetByName("SOTA2")
	CComVariant vBookmarkIndex = m_dwCurrBookmark - 1;
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmarks, L"getByIndex", 1, vBookmarkIndex);
	pdBookmarks->Release();
	pdBookmarks = NULL;
	if (hres != S_OK)
	{
		return hres;
	}
	IDispatch *pdBookmark;
	pdBookmark = vRes.pdispVal;
	if (vBookmarkName != NULL)
	{
		hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"getName", 0);
		if (hres == S_OK)
		{
			*vBookmarkName = vRes;
		}
	}
	if (vText != NULL)
	{
//Set TextRange = Bookmark.GetAnchor()
		IDispatch *pdTextRange;
		hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdBookmark, L"GetAnchor", 0);
		if (hres != S_OK)
		{
			pdBookmark->Release();
			return hres;
		}
		pdTextRange = vRes.pdispVal;
//TextRange.setString("11111111111111")
		hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextRange, L"getString", 0);
		if (hres == S_OK)
		{
			*vText = vRes;
		}
		pdTextRange->Release();
	}
	pdBookmark->Release();
	return S_OK;	
}

HRESULT COOWriterDocument::GetCurrentController(IDispatch **ppdController)
{
	if (m_pDoc == NULL)
		return E_FAIL;
	HRESULT	hres;
	VARIANT vRes;
	hres = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &vRes, m_pDoc->m_pdDoc, L"currentController", 0);
	*ppdController = vRes.pdispVal;
	VariantInit(&vRes);
	return hres;
}
HRESULT COOWriterDocument::GetViewCursor		(IDispatch **ppdCursor)
{
	HRESULT hres;
	IDispatch *pdController;
	VARIANT vRes;
	hres = GetCurrentController(&pdController);
	if (hres != S_OK)
	{
		return hres;
	}
	hres = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &vRes, pdController, L"viewCursor", 0);
	pdController->Release();
	*ppdCursor = vRes.pdispVal;
	VariantInit(&vRes);
	return hres;
}

HRESULT COOWriterDocument::InsertDocumentFromPath(const std::wstring &sPath)
{
//Set oViewCursor = objDocument.CurrentController.getViewCursor()
//oViewCursor.gotoEnd(false)
//Set oTextCursor = objDocument.Text.createTextCursorByRange(oViewCursor.getEnd()) 
//oTextCursor.InsertDocumentFromURL "file:///D|/work/Projects/Com Cloner2/test1.doc", Array() 
    if (m_pDoc == NULL)
    {
        return E_FAIL;
    }
    HRESULT hres;
    VARIANT vRes;
    IDispatch *pdCursor;
    hres = GetViewCursor(&pdCursor);
    if (hres != S_OK)
    {
        return hres;
    }

    CComVariant vParam;
    vParam.vt  = VT_BOOL;
    vParam.boolVal = VARIANT_FALSE;
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdCursor, L"gotoEnd", 1, vParam);
    if (hres != S_OK)
    {
        pdCursor->Release();
        return hres;
    }   
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdCursor, L"getEnd", 0);
    pdCursor->Release();
    if (hres != S_OK)
    {       
        return hres;
    }
    IDispatch *pdRange;
    pdRange = vRes.pdispVal;
    hres = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &vRes, m_pDoc->m_pdDoc, L"Text", 0);
    if (hres != S_OK)
    {
        pdRange->Release();
        return hres;
    }
    IDispatch *pdText;
    pdText = vRes.pdispVal;
    vParam.vt       = VT_DISPATCH;
    vParam.pdispVal = pdRange;
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdText, L"createTextCursorByRange", 1, vParam);
    pdRange->Release();
    pdText->Release();
    VariantInit(&vParam);
    if (hres != S_OK)
    {
        return hres;
    }
    IDispatch *pdTextCursor;
    pdTextCursor = vRes.pdispVal;
    std::wstring sDestination;
    COOApplication::ConvertPathToUrl(sPath, sDestination);
    vParam.vt = VT_ARRAY | VT_VARIANT;
    vParam.parray = NULL;
    CComVariant vParam2 = sDestination.c_str();
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextCursor, L"InsertDocumentFromURL", 2, vParam, vParam2);
    pdTextCursor->Release();
    
    hres = GetViewCursor(&pdCursor);
    if (hres != S_OK)
    {
        return hres;
    }

    vParam.vt  = VT_BOOL;
    vParam.boolVal = VARIANT_FALSE;
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdCursor, L"gotoEnd", 1, vParam);
    if (hres != S_OK)
    {
        pdCursor->Release();
        return hres;
    }   
    vParam = 5;
    hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdCursor, L"BreakType", 1, vParam);
    vParam.vt = VT_BOOL;
    vParam.boolVal = VARIANT_FALSE;

    vParam2 = 0;
    CComVariant vParam3;
    vParam3.vt = VT_DISPATCH;
    vParam3.pdispVal = pdCursor;
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdCursor, L"BreakType", 1, vParam, vParam2, vParam3);
    return hres;
}

HRESULT COOWriterDocument::CreateTextfieldsEnumeration(IDispatch **ppdEnumerator)
{
    if ((m_pDoc == NULL) || (m_pDoc->m_pdDoc == NULL))
    {
        return E_FAIL;
    }
    IDispatch *pdTextFields;
    HRESULT hres;
    VARIANT vRes;
//Set TextFileds = objDocument.getTextFields()
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, this->m_pDoc->m_pdDoc, L"getTextFields", 0);
    if (hres != S_OK)
    {
        return hres;
    }
    pdTextFields = vRes.pdispVal;
//Set Enumeration = TextFileds.createEnumeration()
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextFields, L"createEnumeration", 0);
    pdTextFields->Release();
    if (hres == S_OK)
    {
        *ppdEnumerator = vRes.pdispVal;
    } else 
    {
        *ppdEnumerator = NULL;
    }
    return hres;
}

HRESULT COOWriterDocument::HasMoreElements(IDispatch *pdEnumerator, BOOL &bRes)
{
    if (pdEnumerator == NULL)
    {
        return E_INVALIDARG;
    }
    HRESULT hres;
    VARIANT vRes;
    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdEnumerator, L"hasMoreElements", 0);
    if (hres == S_OK)
    {
        if (vRes.boolVal == VARIANT_TRUE)
        {
            bRes = TRUE;
        } else 
        {
            bRes = FALSE;
        }
    }
    return hres;
}

HRESULT COOWriterDocument::NextElement(IDispatch *pdEnumerator, IDispatch **ppdNextElement)
{
    if (pdEnumerator == NULL)
    {
        return E_INVALIDARG;
    }
    HRESULT hres;
    VARIANT vRes;

    hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdEnumerator, L"nextElement", 0);
    if (hres == S_OK)
    {
        *ppdNextElement = vRes.pdispVal;
    }
    return hres;
}

HRESULT COOWriterDocument::BindToDatasource(CComVariant vDSName, CComVariant vTableName)
{
    if ((m_pDoc == NULL) || (m_pDoc->m_pdDoc == NULL))
    {
        return E_FAIL;
    }
    HRESULT hres;
    VARIANT vRes;
    IDispatch *pdEnumerator;
    hres = CreateTextfieldsEnumeration(&pdEnumerator);
    if (hres != S_OK)
    {
        return hres;
    }
    BOOL    bHasNext;
    hres = COOWriterDocument::HasMoreElements(pdEnumerator, bHasNext);
    if (hres != S_OK)
    {
        pdEnumerator->Release();
        return hres;
    }
    IDispatch *pdTextField;
    IDispatch *pdTextFieldMaster;
    IDispatch *pdAnchor;
//Do While Enumeration.hasMoreElements() 
    while (bHasNext)
    {
//    Set oField = Enumeration.nextElement() 
        hres = COOWriterDocument::NextElement(pdEnumerator, &pdTextField);
        if (hres == S_OK)
        {
//    Set Temp = oField.getTextFieldMaster()    
            hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextField, L"getTextFieldMaster", 0);
            if (hres == S_OK)
            {
                pdTextFieldMaster = vRes.pdispVal;
//    Temp.setPropertyValue "DataColumnName", Mid(s, 2, Len(s)-2)
                hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, pdTextField, L"getAnchor", 0);
                if (hres == S_OK)
                {
                    pdAnchor = vRes.pdispVal;
                    hres = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &vRes, pdAnchor, L"String", 0);
                    if (hres == S_OK)
                    {
                        std::wstring sText = vRes.bstrVal;
                        if ((sText[0] == L'<') && (sText[sText.length() - 1] == L'>'))
                        {

                            wchar_t *szStr = new wchar_t[(sText.length() - 2) + 1];
                            ::memcpy((PVOID) szStr, (PVOID) (sText.c_str() + 1), (sText.length() - 2) * sizeof(wchar_t));
                            szStr[sText.length() - 2] = 0;
                            CComVariant vColumnName = szStr;
                            hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdTextFieldMaster, L"DataColumnName", 1, vColumnName);
                            delete szStr;
//    Temp.setPropertyValue "DataBaseName", "ds"
                            hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdTextFieldMaster, L"DataBaseName", 1, vDSName);
//    Temp.setPropertyValue "DataTableName", "Wordmail"
                            hres = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &vRes, pdTextFieldMaster, L"DataTableName", 1, vTableName);

                        }                    
                    }
                    pdAnchor->Release();
                }
//'   Temp.setPropertyValue "CommandType", 0

            }
            pdTextField->Release();            
        }
        hres = COOWriterDocument::HasMoreElements(pdEnumerator, bHasNext);
    }


//    Set FieldMaster = objDocument.createInstance("com.sun.star.text.FieldMaster.Database")
//'   FieldMaster.Name = "UserEmperor" 
//'   FieldMaster.Content = "sdfsdF"

//    MsgBox oField.getAnchor().String 
//Loop
    
    return S_OK;
}

HRESULT COOWriterDocument::AppActivate(BSTR WindowName, VARIANT Immediate)
{
	return S_OK;
}
HRESULT COOWriterDocument::NextField(short *pResult)
{
	return S_OK;
}
HRESULT COOWriterDocument::AppShow(VARIANT WindowName)
{
//
	return S_OK;
}
HRESULT COOWriterDocument::GetSelStartPos(long *pSelStartPos)
{
// 
	return S_OK;
}
HRESULT COOWriterDocument::SetSelRange(long Pos1, long Pos2)
{
//
	return S_OK;
}
